module graph1_mod
!    use param
    use dal
    use errorHandling

    use omutility_mod
    use omnumerics_mod 

    use omallocation_mod
  

real(single), private :: dy = 2.0





integer :: labelWidth = 4
integer :: axesWidth = 3

real(single) :: symbolSize = 1.5
real(single) :: yLabelOffset = 3.0

integer :: white1, black, red, darkred, blue, green, cyan, orange, yellow, magenta
integer :: indigo, violet, gray, darkgray, brightgray, darkblue, mediumblue, skyblue
integer :: darkgreen, darkyellow, darkcyan, darkmagenta, electricIndigo, electricViolet







interface DrawGraph
    module procedure DrawGraph1, DrawGraph2


end interface DrawGraph







!**************************************************************
!
!*************************************************************






integer :: symbol(10) = (/5, 3, 2, 4, 7, 8, 8, 9, 0, 1/)



integer :: ci(20) = (/1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20/)

contains
!****************************************************************
!
!
!****************************************************************

subroutine DrawGraph1(ndata, xdata, ydata, &
         title, xlabel, ylabel, x1, x2, y1, y2, &
         xmin1, xmax1, ymin1, ymax1, colour, &
         xLog, yLog)
  
    integer, intent(in) :: ndata
    real, intent(in) :: xdata(1 : ndata)
    real, intent(in) :: ydata(1 : ndata)
   

    character(len=*), intent(in) :: title
    character(len=*), intent(in) :: xlabel
    character(len=*), intent(in) :: ylabel
   
    real, intent(in) :: x1, x2, y1, y2
    real, intent(in), optional :: xmin1, xmax1, ymin1, ymax1
    integer, intent(in), optional :: colour
   
    logical, intent(in), optional :: xLog, yLog
    character(10) :: xAxis1, yAxis1, xAxis2, yAxis2
    real ::  xmin, xmax, ymin, ymax
    integer j, colour1
   




    if( present(colour) ) then
        colour1 = colour
    else
        colour1 = red
    end if

       
    xmin =  10000.0
    xmax = -10000.0
    ymin = xmin
    ymax = xmax
             
    do j = 1, ndata
        
        if (xdata(j) .lt. xmin) xmin = xdata(j)
        if (xdata(j) .gt. xmax) xmax = xdata(j)
        if (ydata(j) .lt. ymin) ymin = ydata(j)
        if (ydata(j) .gt. ymax) ymax = ydata(j)
    end do

    if (abs(ymax - ymin) .lt. 1e-10) then
        ymin = ymin - 0.1
        ymax = ymax + 0.1
    end if    

  
    call pgsvp(x1, x2, y1, y2)
    
     
    call pgrnge(xmin, xmax, xmin, xmax)
    call pgrnge(ymin, ymax, ymin, ymax)
    if (present(xmin1)) xmin = xmin1
    if (present(xmax1)) xmax = xmax1
    if (present(ymin1)) ymin = ymin1
    if (present(ymax1)) ymax = ymax1



    call pgsci(black)
    call pgswin(xmin, xmax, ymin, ymax)
    call pgsch(1.0)
  


    


    call pgslw(2)
  
    call pgsch( symbolSize )        

    call pgsci(colour1)





    call pgsci(black)


    call pgsls(1)

    call pgsci(black)  
    call pgslw(axesWidth)


    xAxis1 = "bcts"
    xAxis2 = "n"
 
    yAxis1 = "bcts"
    yAxis2 = "nv"
   

    if(present(xLog) ) then
        if (xLog) then
            xAxis1 = "bctsl"
            xAxis2 = "nl"
        else
            xAxis1 = "bcts"
            xAxis2 = "n"
        end if
    end if

    if(present(yLog) ) then
        if (yLog) then
            yAxis1 = "bctsl"
            yAxis2 = "nvl"
        else
            yAxis1 = "bcts"
            yAxis2 = "nv"
        end if
    end if




    call pgbox(trim(xAxis1), 0.0, 0, trim(yAxis1), 0.0, 0)
    call pgsch(1.0)    



    call pgbox(trim(xAxis2), 0.0, 0, trim(yAxis2), 0.0, 0)

    
    call pgscf(1)

    
    call pgslw(labelWidth)
    call pgscf(2)


    call pgmtxt('T', 0.5, 0.5, 0.5, trim(title))
 
    call pgscf(1)

    call pgsch(1.2) 
    

    call pgmtxt('B', dy, 0.5, 0.5, trim(xlabel))
    call pgmtxt('L', yLabelOffset, 0.5, 0.5, trim(ylabel))
  


    call pgsls(1)
    call pgslw(1)

    call pgsci(black)
end subroutine DrawGraph1





!****************************************************************
!
!
!****************************************************************

subroutine DrawGraph2(ndata, xdata, ydata, yerrors, &
         title, xlabel, ylabel, x1, x2, y1, y2, &
         xmin1, xmax1, ymin1, ymax1, colour, symbolNo, &
         drawLines, xLog, yLog)
  
    integer, intent(in) :: ndata
    real, intent(in) :: xdata(:), ydata(:)
    real, intent(in) :: yerrors(:)
   

    character(len=*), intent(in) :: title
    character(len=*), intent(in) :: xlabel
    character(len=*), intent(in) :: ylabel
   
    real, intent(in) :: x1, x2, y1, y2
    real, intent(in), optional :: xmin1, xmax1, ymin1, ymax1
    integer, intent(in), optional :: symbolNo, colour
   
    logical, intent(in), optional :: xLog, yLog, drawLines
    integer :: colour1


    character(10) :: xAxis1, yAxis1, xAxis2, yAxis2
    real ::  xmin, xmax, ymin, ymax
    integer j
       
    if(present(colour)) then
        colour1 = colour
    else
        colour1 = red

    end if
    xmin =  10000.0
    xmax = -10000.0
    ymin = xmin
    ymax = xmax
             
    do j = 1, ndata
        
        if (xdata(j) .lt. xmin) xmin = xdata(j)
        if (xdata(j) .gt. xmax) xmax = xdata(j)
        if (ydata(j) .lt. ymin) ymin = ydata(j)
        if (ydata(j) .gt. ymax) ymax = ydata(j)
    end do
    if (abs(ymax - ymin) .lt. 1e-10) then
        ymin = ymin - 0.1
        ymax = ymax + 0.1
    end if    

    
    call pgsvp(x1, x2, y1, y2)
   
!   call pgpap( paperWidth, paperHeight / paperWidth)    

     
    call pgrnge(xmin, xmax, xmin, xmax)
    call pgrnge(ymin, ymax, ymin, ymax)
    if (present(xmin1)) xmin = xmin1
    if (present(xmax1)) xmax = xmax1
    if (present(ymin1)) ymin = ymin1
    if (present(ymax1)) ymax = ymax1

  

   call pgsci(black)
 

    call pgswin(xmin, xmax, ymin, ymax)
    call pgsch(1.0)





  call pgsch(symbolSize)        
    call pgsls(2)
   
    if(present(symbolNo) ) then

      

        call pgsci(colour1)  
        call pgpt(nData, xData, yData, symbolNo)
 
 
    else

        call pgsci(colour1)  
        call pgpt(nData, xData, yData, symbol(1))
 
    end if


    call pgsci(colour1)   
    call pgsls(1)           
                     
    call pgerry(ndata, xdata, ydata - yerrors, ydata + yerrors, 1)









    call pgsci(black)

    if(drawLines) then
        call pgsls(1)
        call drawLine(ndata, XData, ydata)
        


        call pgsls(1) 
    end if 
       




    

    call pgsls(1)







    xAxis1 = "bcts"
    xAxis2 = "n"
 
    yAxis1 = "bcts"
    yAxis2 = "nv"
   



    if(present(xLog) ) then
        if (xLog) then
            xAxis1 = "bctsl"
            xAxis2 = "nl"
        else
            xAxis1 = "bcts"
            xAxis2 = "n"
        end if
    end if
    if(present(yLog) ) then
        if (yLog) then
            yAxis1 = "bctsl"
            yAxis2 = "nvl"
        else
            yAxis1 = "bcts"
            yAxis2 = "nv"
        end if
    end if




  
    call pgsci(black)  
    call pgslw(axesWidth)
    call pgbox(trim(xAxis1), 0.0, 0, trim(yAxis1), 0.0, 0)
    call pgsch(0.8)
 
    call pgbox(trim(xAxis2), 0.0, 0, trim(yAxis2), 0.0, 0)

    call pgsch(1.2) 
    call pgslw(labelWidth)

    call pgscf(2)

    call pglabel("", "", title)
    call pgscf(1)



    call pgmtxt('B', dy, 0.5, 0.5, trim(xlabel))
    call pgmtxt('L', yLabelOffset , 0.5, 0.5, trim(ylabel))
  

    call pgslw(1)
  
  
  
    call pgsls(1)
    call pgslw(1)
    call pgsch(0.5)

    call pgsci(black)
 
end subroutine DrawGraph2













!****************************************
! SetUpColourIndice
! Colour indices used to plot fits regions
!***************************************/
subroutine SetUpDefaultColourIndices( )


   

   

    call pgscr(0,   1.0, 1.0, 1.0)           ! white
    white1 = 0



    call pgscr(1,  0.0, 0.0, 0.0)                   ! black
    black = 1

    call pgscr(2,  0.0, 0.0, 1.0)                   ! blue
    blue = 2



    call pgscr(3,  1.0, 0.0, 0.0)                   ! red
    red = 3

   



    call pgscr(4,  0.0, 1.0, 0.0)                   ! green
    green = 4

    call pgscr(5,  0.0, 1.0, 1.0)                   ! Cyan
    cyan = 5

    call pgscr(6,  1.0, 127.0 / 255.0, 0.0)         ! orange
    orange = 6

    call pgscr(7,  1.0, 0.0, 1.0)                   ! Magenta
    magenta = 7

    call pgscr(8,   128.0/255.0, 0.0, 0.0)         ! Dark red
    darkred = 8

    call pgscr(9,  0.0, 0.0, 205.0/255.0)           ! medium blue
    mediumblue = 9

    call pgscr(10,  0.0, 0.0, 139.0/255.0)           ! Dark blue
    darkblue = 10

    call pgscr(11,  135.0/255.0, 206.0/255.0, 1.0)           ! sky blue
    skyblue = 11

    call pgscr(12,  128.0/255.0, 128.0/255.0, 128.0/255.0)           ! gray
    gray = 12

    call pgscr(13,  85.0/255.0, 85.0/255.0, 85.0/255.0)           ! dark gray
    darkgray = 13



    call pgscr(14,  1.0, 1.0, 0.0)                   ! Yellow
    yellow = 14

   
 


    call pgscr(15,  197.0/255.0, 193.0/255.0, 170.0/255.0)           ! medium gray
    brightgray = 15
    call pgscr(16,  128.0/255.0, 128.0/255.0, 0.0)   ! Dark yellow
    darkyellow = 16
    call pgscr(17,  0.0, 128.0/255.0, 0.0)           !Dark green
    darkgreen = 17

    call pgscr(18,  0.0, 128.0/255.0, 128.0/255.0)   ! Dark cyan
    darkcyan = 18

    call pgscr(19,  128.0/255.0, 0.0, 128.0/255.0)   !Dark magenta
    darkmagenta = 19 
    call  pgscr(20,  102.0 / 255.0, 0.0, 1.0)         ! electric indigo
    electricIndigo = 20
    call pgscr(21,  139.0 / 255.0, 0.0, 1.0)         ! electric violet
    electricViolet = 21 
  

    call pgscr(22,  238.0 / 255.0, 130.0/255.0, 238.0/255.0)      ! violet
    violet = 22 
  
    call pgscr(23,  75.0 / 255.0, 0.0, 130.0/255.0)      ! indigo
    indigo = 23 
  







end subroutine SetUpDefaultColourIndices



!*****************************************************
! Subroutine Histogram
!
!*****************************************************
subroutine DrawHistogram(ndata, data, binWidth, &
                 xLabel, title, x1, x2, y1, y2, &
                 xmin1, xmax1)
    integer, intent(in) :: ndata
    real(single), intent(in), dimension(:) :: data
    real(single), intent(inout) :: binWidth
    character(*), intent(in) :: title, xLabel
    real, intent(in) :: x1, x2, y1, y2
    

    real(single), intent(in) ::  xmin1, xmax1
    real(single) :: charSize    
    real(single) :: xmin, xmax, yMax         
    integer noBins, colour1
    integer(int32), pointer:: frequency(:) 
  
    
    character(len=100) :: yLabel 
    integer(int8) :: noLeftBins, noRightBins

   
 


    colour1 = red
    
    xmin = xmin1
    xmax = xmax1
    if (abs(xmin - xmax) .lt. 1e-6) then
        call warning("Unable to draw histogram", "xmin=xmax")
        return
    end if

    charSize = (x2 - x1) * 1.5

   

    
    noBins = int(0.1 + (dble(xmax) - dble(xmin) ) / dble(binWidth)) 



    if (noBins < 3) then
        call warning("omgraph", "No of bins < 3")
        return
    end if




    if( xmin .lt. 0.0 .and. xmax .gt. 0.0) then
    
        noLeftBins = int(xmin / binWidth + 0.00000000001);
        xmin = dble(noLeftBins) * binWidth - binWidth / 2.0;
        noRightBins = int(xmax / binWidth + 0.00000000001);
        xmax = dble(noRightBins) * binWidth + binWidth / 2.0;

    end if




            
       call pgsci(black)
 

    !******************************
    ! Find the maximum bin size
    !******************************

    frequency => AllocateInt32(1, noBins, "frequency")
    frequency = 0


    call computeFrequencies(nData, data, xmin, binWidth, noBins, frequency)



    yMax = maxVal(frequency) * 1.25 * 100.0 / dble(nData)

    yMax = ceiling( yMax / 10.0) * 10.0



    call pgsvp(x1, x2, y1, y2)

  
    call pgwindow(xmin, xmax, 0.0, yMax * real(nData) / 100.0)
     
    call pgsci(colour1)
    
 

    call pgsch(0.8) 
    
    call pgslw(3)    

          
    call pghist(ndata, data, xmin, xmax, noBins, 1)
      
 
  
    deallocate(frequency)  
    nullify(frequency)  

   

    call pgsls(1)



  
    call pgsci(black)   
   
      
    call pgsch(0.8) 
    call pgsfs(2)    
    call pgslw(1)    

    !******************************************************
    ! Convert to frequency
    !
    !*****************************************************

  
    call pgwindow(xmin, xmax, 0.0, yMax)
 


    call pgslw(axesWidth)
    call pgbox("bcts", binwidth, 0, "bcts", 0.0, 0)

    call pgsch(0.7)  
     if(noBins > 20) then
       binwidth = (xmax - xmin) / 20.0
       call pgbox("n", binWidth, 20, "nv", 0.0, 0) 

    else
        call pgbox("n", binWidth, noBins, "nv", 0.0, 0) 
    end if

 
    call pgsch(1.0)
    if(ndata > 0) then
        write(yLabel, "(a, i0, a)") "FREQUENCY (%) (N=", ndata, ")"
    else

        yLabel = "FREQUENCY (%)"

    end if  
    call pgslw(labelWidth)
    call pgscf(2)
 
    call pglabel("", "", title)
    call pgscf(1)



    call pgmtxt('B', dy, 0.5, 0.5, trim(xlabel))

    call pgmtxt('L', yLabelOffset , 0.5, 0.5, trim(ylabel))
  
    call pgslw(1)   

end subroutine DrawHistogram

!**************************************************************************
!
!
!
!**************************************************************************
subroutine computeFrequencies(nData, data, xmin, binWidth, noBins, frequency, refineBinSize)
    integer, intent(in) :: nData
    real(single) :: data(:)
    real(single), intent(in) :: xMin
    real(single), intent(inout) :: binWidth
    integer, intent(in) :: noBins
    integer :: frequency(:)
    logical, intent(in), optional :: refineBinSize
    integer :: i, j, k, maxFrequency
  
    do j = 1, 5
        frequency = 0

        do i = 1, ndata
            k = int((data(i) - xmin) / binWidth + 0.00000001)       
            k = max(1, k)
            k = min(noBins, k)
            frequency(k) = frequency(k) + 1       
        end do
        if( .not. present(refineBinSize)) then
            exit
        end if
        if(.not. refineBinSize) then
            exit
        end if
        maxFrequency = maxVal(frequency) 
        if(real(maxFrequency) / real(ndata) >= 0.1) then
            exit 
        else
         
            binWidth = binWidth / 2.0
        end if
    end do

end subroutine computeFrequencies
!**************************************************
!
!
!************************************************
subroutine drawLine(nData, xData, yData)
    integer, intent(in) :: ndata
    real(single), dimension(:) :: xData, yData
    integer :: j
    call pgmove( xData(1), yData(1))
    do j = 2, nData
        call pgdraw( xData(j), yData(j) )
    end do

end subroutine drawLine

end module graph1_mod
