!************************************************
!
!
! 
!***********************************************
module ompsfprofile1_mod
use dal
use cal
use errorhandling
use omconstants_mod
use omsourceparam_mod  ! In mssllib defines Struct SourceParameters
use omutility_mod
use leastsquares_mod
!use ominterpolate_mod
use integration_mod
use omskybackground_mod
use omphotometrydefs_mod
use omphotometry_mod
use ompsf_mod
use getfilterid_mod
use integration_mod
use gridinterpolation_mod
use splinefit_mod
use brent_mod
use robustscale_mod
use omcoinccorr_mod

implicit none


!*********************************************************************
!
!
!
!
!*********************************************************************

type profileData
    real(single), pointer :: radius(:) => null()
    real(single), pointer :: f(:) => null()
    real(single), pointer :: sumf(:) => null() ! integrated profile inside a circle
    real(single), pointer :: sumreducedf(:) => null() ! integrated reduced profile inside a circle
    real(single), pointer :: df(:) => null()
    real(single), pointer, dimension(:) :: f1=> null(), df1=>null()
    real(single), pointer, dimension(:) :: f2=> null(), df2=>null()
    real(single), pointer, dimension(:) :: f3=> null(), df3=>null()
    real(single), pointer, dimension(:) :: f4=> null(), df4=>null()

    integer :: size = 0
    integer :: size1 = 0
    integer :: size2 = 0
    integer :: size3 = 0
    integer :: size4 = 0

    integer :: sourceNumber
    real(single) :: countsMax
    real(single) :: totalCounts

end type profileData


real(single), pointer :: theimage(:, :) => null()

integer, private :: nRadii1 = 0
real(single), private, pointer, dimension(:) :: radius1 => null(), counts1 => null()

type(OneDSpline),  save :: splineData
!type(PolintStruct), private, save :: polintData

integer, parameter :: maxNumRadii = 51



contains

!************************************************************
!
!
!
!************************************************************
subroutine starprofiles(sources, image, data, k)

    type(SourceParameters), pointer :: sources(:)
    real(single), pointer :: image(:, :)
   
    type(profileData), pointer :: data(:)
    integer, intent(out) :: k
    integer :: i, j, ii
    integer :: nRadii = 51
    logical :: status, status1, status2, status3, status4
    real(single) :: cMax =-1.0

    nRadii = maxNumRadii
    !write(*, *) "L89 ompsfprofile1_mod starprofiles nRadii=", nRadii
    k = 0
    j = 1
    status = .false.
    status1 = .true.
    status2 = .true.
    status3 = .true.
    status4 = .true.




    sourceNumImage => allocate2dInt16(0, ubound(image, 1), 0, ubound(image, 2), "sourceNumImage")
    call refineSourceNumImage(sources, int(size(sources), int16), sourceNumImage)

    do i = 1, size(sources)
       !write(*,*) "----------"
       !write(*, *) "L111 Computing profile for source ", i
       if( .not. associated(data(j)%radius)) then
          data(j)%radius => allocateSingle(1, nRadii, "radius")
          data(j)%f => allocateSingle(1, nRadii, "f")
          data(j)%sumf => allocateSingle(1, nRadii, "sumf") 
          data(j)%sumreducedf => allocateSingle(1, nRadii, "sumreducedf")       
          data(j)%df => allocateSingle(1, nRadii, "df")
          
          data(j)%f1 => allocateSingle(1, nRadii, "f1")
          
          
          data(j)%df1 => allocateSingle(1, nRadii, "df1")
          data(j)%f2 => allocateSingle(1, nRadii, "f2")
          data(j)%df2 => allocateSingle(1, nRadii, "df2")
          data(j)%f3 => allocateSingle(1, nRadii, "f3")
          data(j)%df3 => allocateSingle(1, nRadii, "df3")
          data(j)%f4 => allocateSingle(1, nRadii, "f4")
          data(j)%df4 => allocateSingle(1, nRadii, "df4")

          data(j)%f = -1000.0
          data(j)%f1 = -1.0
          data(j)%f2 = -1.0    
          data(j)%f3 = -1.0
          data(j)%f4 = -1.0
    
          data(j)%sumf = -1000.0         
          data(j)%sumreducedf = -1000.0 

          data(j)%countsMax = -1.0
          data(j)%totalCounts = -1.0
 
       end if
       !write(*,*) "L143 countMax=", data(j)%countsMax, " j=", j, " i=", i

       call starprofile(i,sources(i), image, nRadii, data(j), status)

       if(status) then
          data(j)%sourceNumber = i
          write(errstr,'(a, i0, a, f0.2, a, f0.2, a, f0.2, a, f0.2, a, f0.2)') "Using source ", i, &
               " x=", sources(i)%xpos, " y=", sources(i)%ypos, &
               " signif=", sources(i)%significance, " countsMax=", &
               data(j)%countsMax, " total=", data(j)%totalCounts
          call message(AppMsg, Verbose)
          !write(*,*) "------------------ i=", i, " cMax=", data(j)%countsMax, &
          !     " totalCounts=", data(j)%totalCounts         

          !if (i .eq. 5) then
          !   cMax=data(j)%countsMax
          !   write(*,*) "------------------ i=", i, " cMax=", cMax, &
          !        " totalCounts=", data(j)%totalCounts
          !   do ii=1, nradii
          !      write(*,*) "L147 ii=", ii, " f=", data(j)%f(ii), " sumf=", data(j)%sumf(ii)
          !   enddo
          !endif

          j = j + 1

       end if

    end do
    
    k = j - 1

    !write(*,*) "---------- Finished starprofiles k=", k, " j=",j
    !do i = 1, k
    !   ii=data(i)%sourceNumber
    !   write(*,*) " i=", i, " x=", sources(ii)%xpos, " y=", sources(ii)%ypos, &
    !        " countsMax=", data(i)%countsMax, " total=", data(i)%totalCounts
    !enddo

    deallocate(sourceNumImage)
    nullify(sourceNumImage)
    
end subroutine starprofiles
!************************************************************
!
!
!
!************************************************************
subroutine starprofile(srcNumber, source, image, nRadii, data, status)

    integer, intent(in) :: srcNumber
    type(SourceParameters), intent(in) :: source
    real(single), pointer :: image(:, :)
    integer, intent(in) :: nRadii
    type(profileData), intent(inout) :: data 
    logical, intent(out) :: status
    real(single) :: angle, radius
    real(single) :: x, y, xc, yc, r1, r2, error
    real(single), pointer, dimension(:) :: f=>null()
    real(single) :: meanAv, meanMed, standardDeviation
    real(single) :: xmin, xmax, ymin, ymax, fMax, background, calRadius
    integer :: i, j, k, da, ie
    integer, parameter :: nAngles = 60

    real(single) :: av31, av32 ! sliding average values of last 3 and 5 points
    real(single) :: sumav31, sumav32, d31, d32
    real(single) :: sumreducedav31, sumreducedav32

    integer :: count1, count1c

    real(single) :: innerCounts, innerCountsReduced

    real(single) :: maxRadius, ekoeff, ekoeffReduced, rkoeff, evalue
    real(single) :: evalueReduced
    integer :: indsrc=5

    !*****************************************
    ! 2011-11-02 
    ! a higher-resolution image for each source
    real(single), pointer :: psfImageTest(:, :) => null()
    real(single) :: radius10, xc10, yc10
    integer :: iin, jjn, ii, jj, iic, jjc, ic, jc, iradius
    integer :: nx, ny, lastNradii
    real(single) :: dsumf, ddsumf, dsumfprev, ddsumfprev
    !*****************************************

    data%countsMax=-1.0

    if(source%extensionFlag .eq. 1) then

        status = .false.
        return
    end if

    if(source%confusionFlag .ne. 0) then

        status = .false.
        return

    end if



    if(source%significance < realParameter("minsignificance") .or. &
         source%significance > realParameter("maxsignificance") ) then

       status = .false.
       return
    !else
    !   write(*, *) "L212 extensionflag=", source%extensionFlag
    !   write(*, *) "L213 confusionflag=", source%confusionFlag
    !   write(*, *) "L214 significance=", source%significance
    !   write(*, *) "L214 x=", source%xpos, " y=", source%ypos
    end if

    xmin = real(14 / binx)
    ymin = real(14 / binx)
!    xmin = real(2 / binx)
!    ymin = real(2 / binx)
    xmax = real(size(theimage, 1)) - xmin
    ymax = real(size(theimage, 2)) - ymin

    !write(*, *) "L226 n=", srcNumber, " xmin=", xmin, " xmax=", xmax, " ymin=", ymin, " ymax=", ymax

!***************************************
! 2011-10-06 vny
    xc = source%xpos - 0.5
    !xc = source%xpos 

    if(xc < xmin) then
        status = .false.
        return
    end if

    yc = source%ypos - 0.5
    !yc = source%ypos 

    if(yc < ymin) then
        status = .false.
        return
    end if
 
    if(xc > xmax) then
        status = .false.
        return
    end if
 
    if(yc > ymax) then
        status = .false.
        return
    end if
 


    status = .true.

    da = 360 / nAngles

    !*************************************************************************************
    !
    ! Get the aperture radius from the cal
    !
    !************************************************************************************  
    calRadius =  CAL_omGetApertureRadius()


    r1 = (calRadius + 4.0) / real(binx)
    r2 = (calRadius + 12.0) / real(binx)

    ccdImage => image
!
    !write(*, *) "L269 calRadius=", calRadius
    !write(*, *) "lbound, 1 sourcenumimage=", lbound(sourceNumImage, 1), &
    !     " ubound=", ubound(sourceNumImage, 1)
    !write(*, *) "lbound, 2 sourcenumimage=", lbound(sourceNumImage, 2), &
    !     " ubound=", ubound(sourceNumImage, 2)
    !write(*, *) "lbound, 1 ccdimage=", lbound(ccdImage, 1), " ubound=", ubound(ccdImage, 1)
    !write(*, *) "lbound, 2 ccdimage=", lbound(ccdImage, 2), " ubound=", ubound(ccdImage, 2)


    il = 0
    iu = ubound(sourceNumImage, 1)
    jl = 0
    
    ju = ubound(sourceNumImage, 2)




    background = getBackgroundMode1(sourceNumImage, xc, yc, r1, r2, error)
    
    nullify(ccdImage)


    !write(*, *) "L293 Computed background=", background, " x=", xc, " y=", yc


    f => allocateSingle(1, nAngles, "f")

    data%size = nRadii
    
    !write(*,*) "L301 nRadii=", nRadii

    av31=0.0
    av32=0.0
    sumav31=0.0
    sumav32=0.0
    sumreducedav31=0.0
    sumreducedav32=0.0

    count1=0
    count1c=0

    maxRadius = real(nRadii - 1) / real(binx * 4) 

    !****************************
    ! 2011-11-02
    iin=maxRadius*20
    jjn=maxRadius*20

    iradius=int(maxRadius*10.0+0.5)
    iic=iradius
    jjc=iradius

    ic=int(xc)
    jc=int(yc)

    psfImageTest => allocate2dSingle(0, iin, 0, jjn, "psfimage")
    psfImageTest=-1.0

    nx = size(image, 1)
    ny = size(image, 2)

    !if (srcNumber .eq. 12) then
    !   write(*,*) "L340 iin=", iin, " jjn=", jjn
    !endif

    !*******************************************
    ! Expand the image aroud the source to increase the accuracy
    ! of the cumulative count curve extraction
    !******************************************************
    do ii=0,iin
       do jj=0,jjn
          i=(ii-iic)/10+ic
          j=(jj-jjc)/10+jc

          if (i .gt. 0 .and. i .lt. nx .and. j .gt. 0 .and. j .lt. ny) then
             psfImageTest(ii,jj)= image(i, j)!/100.0
          endif

       enddo
    enddo

    !---------------------------------------------------------------
    ! Run the cicle determining the source profile 
    ! and the background at the end, 
    ! When the background stops changing, the profile measuremet
    ! is terminated; the last radius number is stored in lastNradii
    !----------------------------------------------------------------
    lastNradii=0
    do i = 1, nRadii
        radius = real(i - 1) / real(binx * 4) 
        lastNradii=i
        data%radius(i) = radius

        !write(*,*) "L302 computing counts inside of a circle"

        !***********************************************************
        ! Build a circle with radius "radius" and fint the average
        ! and median values of the image along this circle
        !
        !*********************************************************
        k = 0
        do j = 1, nAngles
            angle = real(j * da) * degreesToRadians
            x = xc + radius * cos(angle)  
            y = yc + radius * sin(angle)  
             
            f(j) = BilinearInterpolation10( image, x, y )
            if(f(j) >= 0.0) then
                k = k + 1
                f(k) = f(j)  

            end if 
        end do
        if( k < 10) then
           
            data%size = i - 1
            exit
        end if
        !*************************************************************
        ! Compute the average around the centre
        !
        !************************************************************
        call ComputeAverage2(f, nAngles, meanAv, standardDeviation)

        !****************************************************
        ! 2011-10-07 use the median value in order to exclude 
        ! counts from bright near-by sources
        !***************************************************
        meanMed=MedianValue(f,nAngles)
        

        !**********************************************
        ! 2011-10-06 vny
        ! data%f(i) = is the source profile 
        ! (containing the beckground for the time being)
        !***********************************************
        data%f(i) = meanMed

        !-------------------------------------------------
        ! Compute the averages of the last 3 values (av32)
        ! and the 3 values preceding them (av31)
        !------------------------------------------------
        if (i .ge. 6) then
           av31=(data%f(i-3)+data%f(i-4)+data%f(i-5))/3.0
        endif
        if (i .ge. 3) then
           av32=(data%f(i)+data%f(i-1)+data%f(i-2))/3.0
        endif

        !*************************************
        data%df(i) = standardDeviation

        !if (srcNumber .eq. indsrc) then
        !   write(*, *) i, ") radius=", radius, &
        !        " std=", standardDeviation, " f(i)=", data%f(i)
        !endif

        !--------------------------------------------------------
        ! Count the number of occasions (count1) when the posterior
        ! value is smaller than the previous one
        ! (the cycle will stop when the number these occasions
        ! reaches 10)
        !--------------------------------------------------------
        if(i > 1) then
            if(data%f(i) .ge. data%f(i - 1) .or. data%f(i) < 0.0) then
               count1=count1+1
               count1c=count1c+1
            else
               count1c=0
            endif
            
            !if(data%f(i) .gt. data%f(i - 1) .or. data%f(i) < 0.0 &
                !.or. standardDeviation .gt. max(1.0, 3.0 * sqrt(mean))) then 
            if (count1 .gt. 10 .or. count1c .gt. 3) then 
                data%size = i - 1
                lastNradii=i
                !if (srcNumber .eq. 12) then
                !   write(*,*) "L369 ***i=", i, " f(i)=",data%f(i), &
                !        " f(i-1)=",data%f(i-1)
                !endif
                exit
            end if
        end if
        !if (srcNumber .eq. indsrc) then
        !   write(*,*) "L372 i=", i, " f=",data%f(i), " count1=", count1, " count1c=", &
        !        count1c
        !endif
    end do 


    !if (srcNumber .eq. indsrc) then
    !   if (i .le. nRadii) then
    !      write(*,*) "---------------------- i=", i, " lastNradii=", lastNradii
    !      write(*,*) "L415  nRadii=", nRadii, " d31=", d31, " d32=", d32, &
    !           " sumav31=", sumav31, " sumav32=", sumav32 
    !      write(*,*) "L423 ekoeff=", ekoeff
    !   endif
    !endif



    !------------------------------------
    ! Compute the background as the minimal value of the last 3 and previous
    ! to the last 3 values;
    ! subtract the background
    !------------------------------------
    av32=min(av31,av32)
    if (av31 .lt. av32) then
       av32=av31
       !sumav32=sumav31
    endif

    !write(*,*) "L392 using the minimal value as background=", av32
    do i = 1, lastNradii!data%size
       data%f(i)=data%f(i)-av32
    enddo

    data%countsMax=data%f(1)

    !*************************************
    ! 2011-11-03 vny
    ! repeat the cycle calculating the counts 
    ! of all pixels inside of the circle
    !*************************************
    !---------------------------------------------------------------
    ! Run the cicle determining the cumulative counts 
    ! Calculate this only up to the last i found in the previous cycle
    ! (lastNradii)
    !----------------------------------------------------------------
    dsumfprev=0.0
    ddsumfprev=0.0
    dsumf=0.0
    ddsumf=0.0
    do i = 1, lastNradii!nRadii
       radius = real(i - 1) / real(binx * 4) 
       
       innerCounts=CountsInCircle(image, xc, yc, radius, maxRadius, &
            srcNumber, psfImageTest, av32)
       
       innerCountsReduced=CountsInCircleReduced(image, xc, yc, radius, maxRadius, &
            srcNumber, psfImageTest, av32)

       data%sumf(i)=innerCounts
       data%sumreducedf(i)=innerCountsReduced
       
       if (i .ge. 6) then
          sumav31=(data%sumf(i-3)+data%sumf(i-4)+data%sumf(i-5))/3.0
          sumreducedav31=(data%sumf(i-3)+data%sumf(i-4)+data%sumf(i-5))/3.0
          d31=float(i-3+i-4+i-5)/3.0
       endif
       if (i .ge. 3) then
          sumav32=(data%sumf(i)+data%sumf(i-1)+data%sumf(i-2))/3.0
          sumreducedav32=(data%sumf(i)+data%sumf(i-1)+data%sumf(i-2))/3.0
          d32=float(i+i-1+i-2)/3.0
       endif
 
       if (i .gt. 1) then
          dsumf=data%sumf(i)-data%sumf(i-1)
          ddsumfprev=ddsumf
          ddsumf=dsumf-dsumfprev
          dsumfprev=dsumf
       else
          dsumf=0.0
       endif


       !if (srcNumber .eq. indsrc) then
       !   write(*,*) "L554 i=", i, " sumf=", data%sumf(i),  &
       !        " sumreducedf=", data%sumreducedf(i), " dsumf=", &
       !        dsumf, " ddsumf=", ddsumf, " ddsumfprev=", ddsumfprev
       !endif

    end do


    !*************************************
    ! 2011-10-31 vny
    ! av32 is the background
    ! 
    !*************************************
    !if (srcNumber .eq. indsrc) then
    !   write(*,*) "---------------------- i=", i, " lastNradii=", lastNradii
    !   write(*,*) "L567 sumav32=", sumav32, " sumav31=", sumav31, " d32=", d32, &
    !        " d31=", d31
    !   write(*,*) "L569 sumredav32=", sumreducedav32, " sumredav31=", sumreducedav31
    !endif

    ! extrapolate the curve of growth
    ekoeff=(sumav32-sumav31)/(d32-d31)
    ekoeffReduced=(sumreducedav32-sumreducedav31)/(d32-d31)


    !if (i .le. nRadii) then 
    !   do ie=i,nRadii
    !      rkoeff=float(ie-i+1)
    !      evalue=ekoeff*rkoeff+data%sumf(i-1)
    !      evalueReduced=ekoeffReduced*rkoeff+data%sumreducedf(i-1)
    !      if (srcNumber .eq. indsrc) then
    !         write(*,*) "L582  ie=", ie, " evalue=", evalue, " evalueReduced=", evalueReduced
    !      endif
    !   enddo
    !else
    !   evalue=data%sumf(nRadii)
    !   evalueReduced=data%sumreducedf(nRadii)
    !endif

    if (abs(ddsumf) .gt. abs(dsumf)) then
       do ie=1,4
          ddsumf=ddsumf/2.0
          if (abs(ddsumf) .lt. abs(dsumf)) then
             exit
          endif
       enddo
    endif
    !if (srcNumber .eq. indsrc) then
    !   write(*,*) "L616 dsumf=", dsumf, " ddsumf=", ddsumf
    !endif
    
    if (i .le. nRadii) then 
       do ie=i,nRadii
          dsumf=dsumf+ddsumf
          ddsumf=ddsumf/2.0
          !rkoeff=1.0!float(ie-i+1)
          evalue=data%sumf(ie-1)+dsumf
          evalueReduced=evalue/100.0
          data%sumf(ie)=evalue
          data%sumreducedf(ie)=evalueReduced
          !if (srcNumber .eq. indsrc) then
          !   write(*,*) "L582  ie=", ie, " evalue=", evalue, " evalueReduced=", evalueReduced
          !endif
       enddo
    else
       evalue=data%sumf(nRadii)
       evalueReduced=data%sumreducedf(nRadii)
    endif




    !--------------------------------------------------------------
    ! Store evalueReduced to later put it into the output table
    ! as the number of counts inside of the circle of radius nRadii
    !---------------------------------------------------------------
    data%totalCounts=evalueReduced
    !-------------------------------------------------------
    ! Normalise the cumulative counts 
    ! by using the extrapolated value at the distance  nRadii
    !-------------------------------------------------------
    do i = 1, nRadii
       if (data%sumf(i) .gt. -10.0) then
          data%sumf(i)=data%sumf(i)/evalue
       else
          data%sumf(i)=-1.0
       endif
    enddo    


    !*******************************
    ! 2011-11-02
    deallocate(psfImageTest)


    !if (srcNumber .eq. indsrc) then
    !   write(*,*) "L412 nRadii=", nRadii
    !   write(*,*) "L413  sumav32=",sumav32, &
    !        " sumav31=", sumav31
    !   write(*,*) "--------------"
    !   do i = 1, nRadii
    !      write(*,*) "L430 i=", i, " sumf=",data%sumf(i)     
    !   enddo
    !   write(*,*) "--------------"
    !endif


    if(data%size < maxNumRadii / 2 ) then

        status = .false.

    else
        status = .true.       
        !*************************************
        ! 2011-10-06 vny
        fmax = data%f(1)
        !*************************************
        if(fMax > 1.0) then
           do i = 1, nRadii
              if (data%f(i) .gt. -10.0) then
                 data%f(i) = data%f(i) / fMax
                 data%df(i) = data%f(i) / fMax
              else
                 data%f(i) = -1.0
                 data%df(i) = -1.0
              endif
         enddo

        end if

    end if

    deallocate(f)

    !-----------------------------------------------------------
    ! evalueReduced is the number of counts in the maximal circe
    ! fMax is the maximal value of counts at the centre of the source
    !------------------------------------------------------------   


end subroutine starprofile


!***************************************************************
!
!
!
!
!**************************************************************
subroutine computeAverageProfile(data, ndata, xMean, yMean, error, nRadii, nRadiiOut)
    type(profileData), pointer :: data(:)
    integer, intent(in) :: nData
   
    real(single), pointer, dimension(:) :: xMean, yMean, error
    integer, intent(in) :: nRadii
    integer, intent(out) :: nRadiiOut
  
    real(single), pointer, dimension(:) :: f => null(), df => null()
    real(single) :: mean, standardDeviation    
    integer :: i, j, k, n, nCount
  

    

    if( .not. associated(xMean)) then
        xMean => allocateSingle(1, nRadii, "x")
        yMean => allocateSingle(1, nRadii, "y")
        error => allocateSingle(1, nRadii, "error")
    end if


    f => allocateSingle(1, nData, "f")
    df => allocateSingle(1, nData, "df")

 


 
   n = 0

   !write(*,*) "L460 Compute average profile nRadii=", nRadii
   !write(*,*) "=============================================="
    do i = 1, nRadii  ! Loop through all the radii
       !write(*, *) i, ") computing mean profile for radius ", real(i - 1) / real(binx * 4) 
        nCount = 0
        do j = 1, nData
            k = data(j)%size
          
            if(i > k) then
                cycle
            end if
            !*********************
            ! 2011-10-08 vny
            !if( data(j)%f(i) .le. 0.0) exit
            !if( data(j)%f(i) .lt. 0.0) exit
            !***************************
            nCount = nCount + 1 
            f(nCount) = data(j)%f(i) 
            df(nCount) = data(j)%df(i)
           
        end do
        !write(*, *) i, ") number of points to average for radius ", i, " =", nCount
        if(ncount > 0) then
           n = n + 1
                      
           call computeTheMean(nCount, f, df, mean, standardDeviation)
                     
           xMean(n) = real(i - 1) / real(binx * 4) 
           yMean(n) = mean
           error(n) = standardDEviation
           !write(*, *) "i=", i, " n= ", n, "  xmean=", xmean(n), " ymean=", &
           !     ymean(n), " error=", error(n), " nCount=", nCount
        else
            exit
        end if 
           

    end do    

    nradiiOut = n
    !write(*, *) "computeaverageprofile line 1370 - nradiiOut=", n
    deallocate(f, df)

end subroutine computeAverageProfile



!***************************************************************
!
!
!
!
!**************************************************************
subroutine computeAverageProfile2(data, ndata, xMean, yMean, ysumMean, error, nRadii, nRadiiOut)
    type(profileData), pointer :: data(:)
    integer, intent(in) :: nData
   
    real(single), pointer, dimension(:) :: xMean, yMean, ysumMean, error
    integer, intent(in) :: nRadii
    integer, intent(out) :: nRadiiOut
  
    real(single), pointer, dimension(:) :: f => null(), sumf => null(), df => null()
    real(single) :: mean, summean, standardDeviation    
    integer :: i, j, k, n, nCount
  

    

    if( .not. associated(xMean)) then
        xMean => allocateSingle(1, nRadii, "x")
        yMean => allocateSingle(1, nRadii, "y")
        ysumMean => allocateSingle(1, nRadii, "ysum")
        error => allocateSingle(1, nRadii, "error")
    end if


    f => allocateSingle(1, nData, "f")
    sumf => allocateSingle(1, nData, "sumf")
    df => allocateSingle(1, nData, "df")

 


 
   n = 0

   !write(*,*) "L460 Compute average profile nRadii=", nRadii
   !write(*,*) "=============================================="
    do i = 1, nRadii  ! Loop through all the radii
       !write(*, *) i, ") computing mean profile for radius ", real(i - 1) / real(binx * 4) 
        nCount = 0
        do j = 1, nData
            k = data(j)%size
          
            if(i > k) then
                cycle
            end if
            !*********************
            ! 2011-10-08 vny
            !if( data(j)%f(i) .le. 0.0) exit
            !if( data(j)%f(i) .lt. 0.0) exit
            !***************************
            nCount = nCount + 1 
            f(nCount) = data(j)%f(i) 
            sumf(nCount)=data(j)%sumf(i)
            df(nCount) = data(j)%df(i)
           
        end do
        !write(*, *) i, ") number of points to average for radius ", i, " =", nCount
        if(ncount > 0) then
           n = n + 1
                      
           !call computeTheMean(nCount, f, df, mean, standardDeviation)
           call computeTheMean2(nCount, f, sumf, df, mean, summean, standardDeviation)                     
           xMean(n) = real(i - 1) / real(binx * 4) 
           yMean(n) = mean
           ysumMean(n)=summean
           error(n) = standardDEviation
           !write(*, *) "i=", i, " n= ", n, "  xmean=", xmean(n), " ymean=", &
           !     ymean(n), " error=", error(n), " nCount=", nCount
        else
            exit
        end if 
           

    end do    

    nradiiOut = n
    !write(*, *) "computeaverageprofile line 1370 - nradiiOut=", n
    deallocate(f, df)

end subroutine computeAverageProfile2



!*********************************************************************
!
!
!
!*********************************************************************
subroutine getPsf(psfImage, source)

    type(sourceParameters), intent(out) :: source
   
    real(single), pointer :: psfImage(:, :)
    integer :: psfNx, psfNy
    real(single) :: fMax
  
    psfnx = 20 ! 25 pixel width
    psfny = 20 ! 25 pixel height


    !*************************************************
    !
    ! Get the point-spread function image
    !
    !*************************************************
    call CreatePSFImage21(psfNx, psfNy, psfImage, &
          1024.5_double, 1024.5_double,                     &
          ratio=0.5_double, bin=binX, res=1) 

    fMax = maxval(psfImage)

    psfImage = psfImage * 100.0 / fMax + backgroundCountsMode

    psfNx = size(psfImage, 1)
    psfNy = size(psfImage, 2)

   
    call SaveImage( psfimage, int(psfnx, int16), int(psfny, int16), "callpsfimage.fits", "psf image")
      

    source%xPos = real( psfNx ) / 2.0 + 0.5
    source%yPos = real( psfNy ) / 2.0 + 0.5

    source%significance = 50.0
    source%extensionFlag = 0
    source%qualityFlag = 0
    source%confusionFlag = 0
 
 !   background = ComputeBackgroundMode2(psfImage, int(psfnx, int16), int(psfny, int16), 3.0, sigma)
 !   write(*, *) "psf Background mode=", background


end subroutine getpsf

!********************************************************************
!
! Compute the average between angles theta1 and theta2 degrees
!
!*******************************************************************
subroutine average(f, df, theta1, theta2, nAngles, radius, background, xc, yc, status)
    real(single), intent(out) :: f, df
    real(single), intent(in) :: theta1, theta2     
    integer, intent(in) :: nAngles 
    real(single), intent(in) :: radius, background, xc, yc
    logical, intent(out) :: status
    real(single), pointer :: f1(:)=>null()
    real(single) :: mean, standardDeviation, x, y, angle, da
    real(single) :: nx, ny
   
    integer :: i, k
  
    nx = real(ubound(theimage, 1))
    ny = real(ubound(theimage, 2))

    da = (theta2 - theta1) / real(nAngles - 1)

    f1 => allocateSingle(1, nAngles, "f")
    
    !***********************************************************
    !
    !
    !*********************************************************
    k = 0
    do i = 1, nAngles
        angle = (theta1 + real(i - 1) * da) * degreesToRadians
        x = xc + radius * cos(angle)
        if(x < 0.5 .or. x > nx) then
            cycle 

        end if  
        y = yc + radius * sin(angle)  
        if(y < 0.5 .or. y > ny) then
            cycle 
        end if 
 
        k = k + 1

        f1(k) = BilinearInterpolation10(theimage, x, y)

    end do

    !*************************************************************
    ! Compute the average around the centre
    !
    !************************************************************
    if(k < 1) then
        status = .false.
        f = -1.0
        df = -1.0
    else
   
        call ComputeAverage2(f1, k, mean, standardDeviation)
        if(standardDeviation .gt. max(1.0, 3.0 * sqrt(mean)) ) then
            status = .false.
            f = -1.0
            df = -1.0
        else
            status = .true. 
            f = mean - background
            df = standardDeviation
        end if
    end if
    deallocate(f1)


end subroutine average

!***************************************************************
!
!
!****************************************************************
real(double) function psfIntegrationFunction( x )
    real(double), intent(in) :: x



    psfIntegrationFunction = linearInterpolation(abs(real(x)), &
                  radius1(1 : nradii1), counts1(1 : nradii1))
 

end function psfIntegrationFunction



!********************************************************
!
!
!********************************************************
real(single) function linearInterpolation(xa, x, y)
    real(single), intent(in) :: xa
    real(single) ::  x(:), y(:)
    real(single) :: dx, x1, x2, f1, f2
    integer :: i, size1

    size1 = size(x)

    if(xa < x(1) .or. xa > x(size1) ) then
!        write(*, *) "linearinterpoltion, returning -1, xa=", xa, &
!              " x(1)=", x(1), " size1=", size1, " x(size1)=", x(size1)
        linearInterpolation = -1.0
        return

    end if

    if(abs(xa - x(1)) .lt. 1e-6) then
        linearInterpolation = y(1)
        return
    end if

    if( abs(xa - x(size1)) .lt. 1e-6) then
        linearInterpolation = y(size1)
        return
    end if


    do i = 1, size1 - 1
        if( xa.ge. x(i) .and. xa .le. x(i + 1)) then
            x1 = x(i)
            f1 = y(i)  
            x2 = x(i + 1)
            f2 = y(i + 1)
            exit
        end if 
   
    end do

    dx = x2 - x1

    linearInterpolation =  (1.0 - (xa - x1) / dx) * f1 + &
                           (1.0 - (x2 - xa) / dx) * f2 


!    if(linearInterpolation < 0.0) then
!         write(*, *) "xa=", xa, " x1=", x1, " x2=", x2, " f1=", f1, " f2=", f2, " dx=", dx
!    end if

end function linearInterpolation

!***********************************************************
!
!
!**********************************************************
logical function imageOkay(image)
    real(single) :: image(:, :)
    integer :: il, iu, jl, ju, i, j
    real(single) :: maxHeight

    il = lbound(image, 1)
    iu = ubound(image, 1)
    jl = lbound(image, 2)
    ju = ubound(image, 2)

    maxHeight = backgroundCountsMode + max(4.0_double, 5.0 * sqrt(backgroundCountsMode))
maxHeight = 0.0

    do i = il, iu
        do j = jl, ju
            if(image(i, j) .gt. maxHeight) then
                imageOkay = .false.
                return
            end if

        end do
    end do

    imageOkay = .true.

end function imageOkay



!******************************************************************
!
!
!*****************************************************************
subroutine computeFWHM(nData, x, y, fwhm)
    integer, intent(in) :: nData
    real(single), dimension(:) :: x, y
    real(single), intent(out) :: fwhm
    real(double) :: ax, bx, tol, xmin, fmin
    integer :: ndata1

    integer i


    nData1 = count(x(1 : nData)  >= 0.0)
    !**********************************************************
    !
    !
    !**********************************************************
    call ConstructSpline(nData1, x, y, splineData)


    !write(*, *) "computeFWHM- the data, ndata=", ndata, " ndata1=", ndata1
    !do i = 1, ndata1
    !   write(*, *) i, ") x=", x(i), " y=", y(i)
    !end do

    ax = 1.0_double / dble(binX)
    bx = dble( 8 / binx )
 
    xmin = (ax + bx) / 2.0_double
    tol = 0.0000000001_double

  


    fMin = rtbis(ax, bx, tol, xmin, fwhmFunction)



    fwhm = xmin * 2.0

   


    call DestructSpline( splineData )




end subroutine computeFWHM






!*************************************************************
!
!
!
!*************************************************************
real(double) function cumulativeCountsFunction(x)
    real(double), intent(in) :: x
 
  
    cumulativeCountsFunction = InterpolateSpline(x, splineData)



end function cumulativeCountsFunction



!*************************************************************
!
!
!
!*************************************************************
real(double) function fwhmFunction(x)
    real(double), intent(in) :: x
  
  
    fwhmFunction = 0.5_double - InterpolateSpline(x, splineData)



end function fwhmFunction


!*************************************************************
!
!
!*************************************************************
real(double) function fwhmFunction1(x)
    real(double), intent(in) :: x
    

    fwhmFunction1 = min(1.0_double, 0.5_double - InterpolateSpline(x, splineData))

!    write(*, *) "x=", x, " interpolated value=", InterpolateSpline(x, splineData)

end function fwhmFunction1







!*****************************************************************************
!
!
!
!*****************************************************************************





real(double)  FUNCTION rtbis(x1, x2, TOL, xmin, func)
    real(double), intent(in) :: x1, x2, tol
    real(double), intent(out) :: xmin

interface 

    function func(x) result(res)

    use dal
        real(double), intent(in) :: x
        real(double) :: res
    end function func
end interface    


    real(double) :: dx, f, fmid, xmid, rtb
    integer :: j
    integer, parameter :: jMax = 100

    f = func( x1 )
    fmid = func(x2)

    if(f * fmid .ge. 0.0) then

        write(*, *) "Root must be bracketed in rtbis"
        rtbis = 0.0
        return
    end if

    if( f < 0.0) then
        dx = x2- x1
        rtb = x1
    else
        dx = x1 - x2
        rtb = x2
    end if

    do j = 1, jmax
        dx = dx * 0.5
        xmid = rtb + dx 
        fmid = func(xmid)
        if( fmid <= 0.0) then
            rtb = xmid
        end if
        if( abs(dx) .lt. tol .or. fmid .eq. 0.0) then
            rtbis = fmid
            xmin = rtb
         
            return
        end if
    end do

    rtbis = 0.0
    write(*, *) "Too many iterations in rtbis"



end function rtbis


!*********************************************************
!
!
!*********************************************************  
subroutine computePSFImage(nRadiiIn, radius, counts, psfImageOut, psfGridOut)
    integer, intent(in) :: nradiiIn
    real(single), dimension(:) :: radius, counts
   
    real(single), optional, pointer :: psfImageOut(:, :)
    type(gridInterpolant), intent(out), optional :: psfGridOut
    real(single), pointer :: psfImage(:, :) => null()
    integer :: nx, ny, nRadii
    real(single) :: dx, dy, xc, yc, x1, y1, r, maxRadius, f4
          
    real(single), pointer, dimension(:) :: x => null(), y => null()
    real(double) :: xl, xu
    type(oneDSpline) :: splineData

    integer :: i, j
!    type(polintstruct) :: polintdata

!    call polintConstruct(nRadii, radius, counts, polintdata)



     write(errstr,'(a)') "Computing psf image"
     call message(AppMsg, Verbose)
  

    nRadii = count( radius(1 : nRadiiIn) > -0.1)

    nRadii1 = nRadii
    counts1 => allocateSingle(1 , nradii, "counts1")
    radius1 => allocateSingle(1 , nradii, "radius1")


    !write(*, *) "computepsfimage L913, nRadiiIn=", nRadiiIn
    !write(*, *) "nRadii=", nradii
    

    radius1(1 : nradii) = radius(1 : nRadii)
    counts1(1 : nradii) = counts(1 : nRadii)
  


    maxRadius = radius(nRadii) 
    nx = 2 * (20 / binx) + 1
    ny = nx

    xc = real(nx) / 2.0 + 0.5
    yc = real(ny) / 2.0 + 0.5
    psfImage => allocate2dSingle(0, nx - 1, 0, ny - 1, "psfimage")

    psfImage = 0.0

!    write(*, *) "computepsfimage, nradii=", nRadii, " maxradius=", maxRadius


    !do i = 1, nRadii
    !    write(*, *) i, ") radius=", radius(i), " height=", counts(i)
    !end do


    call constructSpline(nRadii, radius, counts, splineData)


    do i = 0, nx - 1
        x1 = real(i + 1) 
        
        dx = x1 - xc
        if(abs(dx) > maxRadius) cycle
   
        do j = 0, ny - 1
            y1 = real(j + 1)
         
            dy = y1 - yc
            if(abs(dy) > maxRadius) cycle
       
            r = sqrt(dx * dx + dy * dy)
            if(r > maxRadius) cycle



            xl = dble(r) - 0.5_double
            xu = dble(r) + 0.5_double


!            f1 = linearInterpolation(r, radius(1 : nradii), counts(1 : nradii))
!            f2 = qgauss(xl, xu, psfIntegrationFunction)       
!            f3 = polintInterpolate2(r, nRadii, radius, counts, error)




            f4 = interpolateSpline(r, splineData)



!            write(*, *) "r=", r, " f1=", f1, " f2=", f2, " f3=", f3, " f4=", f4
 
           psfImage(i, j) = f4
!            write(*, *) "psfimage(", i, ",", j, ")=", psfimage(i, j)
        end do
    end do




    call destructSpline(splineData)




    if(present(psfImageOut)) then
        psfImageOut => psfImage
  
    end if


  
    if( .not. present(psfGridOut) ) then
       
        return
    end if


    !**********************************************************************
    !
    !
    !
    !**********************************************************************



    x => allocateSingle(1, nx, "x")
    y => allocateSingle(1, ny, "y")

    do i = 1, nx
        x(i) = real(i) - xc
        y(i) = real(i) - yc

    end do

      
    call GridInterpolantConstruct( nx, ny, x, y, psfImage, psfGridOut)




    if( .not. present(psfImageOut)) then
        deallocate(psfImage)
        nullify(psfImage)
  
    end if


    deallocate(x, y)
    deallocate(counts1, radius1)
    nullify(counts1, radius1)




end subroutine computePSFImage

!*****************************************************************
!
!
!*****************************************************************
subroutine computetheMean(size, f, df, mean, standardDeviation)
    integer, intent(inout) :: size
    real(single), dimension(:) :: f, df
    real(single), intent(out) :: mean, standardDeviation                

    real(single) :: median, snValue, std
    integer :: i, k


    if(size < 5) then
        call ComputeAverage(f, df, size, mean, standardDeviation)
        return
    end if

    snValue = sn(f, size)
    median = MedianValue(f(1 : size), size)
    call ComputeAverage(f, df, size, mean, standardDeviation)
     
    !write(*, *) "computethe mean- input size=", size, " mean=", mean, &
    !     " std=", standardDeviation, &
    !     " median=", median, " sn=", snValue

    k = 0
 
    std = 2.0 * snValue

    do i = 1, size
        if ( abs(median - f(i)) .le. std) then
            k = k + 1
            f(k) = f(i)
            df(k) = df(i)
        end if
    end do

    if(k >= size / 2) then
        size = k
        call ComputeAverage(f, df, size, mean, standardDeviation)
    end if

    !write(*, *) "L1088 computethemean- output size=", size, &
    !     " mean=", mean, " std=", standardDeviation
 

end subroutine computeTheMean


!**********************************
! ComputeAverage
! Calculates the weighted average and
! standard deviation
!**********************************
subroutine ComputeAverage2noNegative(data, size, mean, standardDeviation)
    integer(int32), intent(in) :: size        
    real(single) :: data(:)    
    real(single), intent(out) :: mean, standardDeviation
 
    real(double) :: sum2, nsum
    real(double) :: d, difference
    
    integer i



    if (size .eq. 1) then
        mean = data(1)
        standardDeviation = 0.0
        return
    end if

    mean=0.0
    nsum=0.0d0
    do i = 1, size
       if (data(i) .ge. 0.0) then
          mean=mean+data(i)
          nsum=nsum+1.0
       endif        
    end do
    if (nsum .gt. 1.0) then
       mean=mean/nsum
    endif
    
    sum2 = 0.0d0 
    nsum =0.0d0
    do i = 1, size
        difference = data(i) - mean             
        sum2 = sum2 + difference * difference        
    end do
    
    d = dble(size - 1)
    
    standardDeviation = dsqrt(sum2 / d-1.0d0) ! The standard deviation

end subroutine ComputeAverage2noNegative

!*****************************************************************
!
!
!*****************************************************************
subroutine computetheMean2(size, f, sumf, df, mean, summean, standardDeviation)
    integer, intent(inout) :: size
    real(single), dimension(:) :: f, sumf, df
    real(single), intent(out) :: mean, standardDeviation, summean                

    real(single) :: median, snValue, std
    integer :: i, k
    real(single) :: sumstandardDeviation

    if(size < 5) then
        call ComputeAverage(f, df, size, mean, standardDeviation)
        call ComputeAverage2noNegative(sumf, size, summean, sumstandardDeviation)
        return
    end if

    snValue = sn(f, size)
    median = MedianValue(f(1 : size), size)
    call ComputeAverage(f, df, size, mean, standardDeviation)
    call ComputeAverage2noNegative(sumf, size, summean, sumstandardDeviation)     

    !write(*, *) "computethe mean- input size=", size, " mean=", mean, &
    !     " std=", standardDeviation, &
    !     " median=", median, " sn=", snValue

    k = 0
 
    std = 2.0 * snValue

    do i = 1, size
        if ( abs(median - f(i)) .le. std) then
            k = k + 1
            f(k) = f(i)
            df(k) = df(i)
        end if
    end do

    if(k >= size / 2) then
        size = k
        call ComputeAverage(f, df, size, mean, standardDeviation)
    end if

    !write(*, *) "L1088 computethemean- output size=", size, &
    !     " mean=", mean, " std=", standardDeviation
 

end subroutine computeTheMean2



!***********************************************************
!
!
!***********************************************************
real(single) Function  BilinearInterpolation10(imagePtr, x, y)
    real(single), pointer :: imagePtr(:, :)
    real(single), intent(in) :: x, y

    real(single) :: xl, xu, yl, yu
    real(single) :: u, t
    integer j0, k0, j1, k1
    integer nx, ny

    nx = size(imagePtr, 1)
    ny = size(imagePtr, 2)
    xl = 0.0
    yl = 0.0
    xu = real(nx)
    yu = real(ny)

    if (x .gt. xl .and. x .lt. xu .and. & 
        y .gt. yl .and. y .lt. yu) then

        

        j0 = max(0, int(x - 0.5))
        j0 = min(nx - 1, j0)
      
        k0 = max(0, int(y - 0.5))
        k0 = min(ny - 1, k0)


        
        j1 = j0 + 1
        k1 = k0 + 1

        if (j1 .ge. nx) then
            t = 0.0
            j1 = nx - 1
        else        
            t = max(0.0, x - real(j0) - 0.5)
        end if
        if (k1 .ge. ny) then
            k1 = ny - 1
            u = 0.0
        else  
            u = max(0.0, y - real(k0) - 0.5)
        end if

        if( isNullValue(imagePtr(j0, k0)) ) then

            BilinearInterpolation10 = -1.0
            return
        end if

        if( isNullValue(imagePtr(j1, k0)) ) then

            BilinearInterpolation10 = -1.0
            return
        end if

        if( isNullValue(imagePtr(j1, k1)) ) then

            BilinearInterpolation10 = -1.0
            return
        end if

        if( isNullValue(imagePtr(j0, k1)) ) then

            BilinearInterpolation10 = -1.0
            return
        end if



    !   write(*, *) "j0=", j0, " k0=", k0
    !   write(*, *) "j1=", j1, " k0=", k0
    !   write(*, *) "j1=", j1, " k1=", k1
    !   write(*, *) "j0=", j0, " k1=", k1
    !   write(*, *) "t=", t, " u=", u


      
        BilinearInterpolation10 = &
              (1.0 - t) * (1.0 - u) * imagePtr(j0, k0) + &
               t * (1.0 - u) * imagePtr(j1, k0) + &
               t * u * imagePtr(j1, k1) + &
               (1.0  - t) * u * imagePtr(j0, k1)



        

        else
            BilinearInterpolation10 = -1.0
    end if


end function BilinearInterpolation10


!****************************************************
! Subroutine RealImage
!****************************************************
subroutine SaveImageTest(image, nx, ny, fileName, heading)

    integer, intent(in) :: nx, ny    
 
    real (single), intent(in), dimension(0 : nx - 1, 0 : ny - 1) :: image
    character(*), intent(in) :: fileName
    character(*), intent(in) :: heading
    type(DataSetT)       :: imageSet 
    real (single), dimension(:, :), pointer :: imageP
    type(ArrayT)         :: imageArray         ! array for background
   
 !write(*, *) "omutility_mod saverealimage1 line 1300, saving to file ", trim(fileName)
    imageSet = DataSet(trim(fileName), Create)

    if (hasBlock(imageSet, "PRIMARY")) then
        call deleteBlock( imageSet, "PRIMARY" );
    end if



    if (heading .eq. "test") then

    end if  




    imageArray = addArray(imageSet, "PRIMARY", real32, shape(image))

    imageP => real32Array2Data(imageArray)

    imageP = image

    call release(imageArray)

    call release(imageSet)
 


!write(*, *) "Leaving saverealimage1"
   
end subroutine SaveImageTest

!****************************************************
! Subroutine RealImage
!****************************************************
subroutine SaveImageTest2(image, i1,i2,j1,j2, fileName, heading)

    !integer, intent(in) :: nx, ny    
    integer, intent(in) :: i1, i2,j1,j2   

    !real (single), intent(in), dimension(0 : nx - 1, 0 : ny - 1) :: image
    real (single), intent(in), dimension(i1 : i2, j1 : j2) :: image
    character(*), intent(in) :: fileName
    character(*), intent(in) :: heading
    type(DataSetT)       :: imageSet 
    real (single), dimension(:, :), pointer :: imageP
    type(ArrayT)         :: imageArray         ! array for background
   
 !write(*, *) "omutility_mod saverealimage1 line 1300, saving to file ", trim(fileName)
    imageSet = DataSet(trim(fileName), Create)

    if (hasBlock(imageSet, "PRIMARY")) then
        call deleteBlock( imageSet, "PRIMARY" );
    end if



    if (heading .eq. "test") then

    end if  




    imageArray = addArray(imageSet, "PRIMARY", real32, shape(image))

    imageP => real32Array2Data(imageArray)

    imageP = image

    call release(imageArray)

    call release(imageSet)
 


!write(*, *) "Leaving saverealimage1"
   
end subroutine SaveImageTest2

!***********************************************************
! CountsInCircle 2011-10-19 VNY
! To compute the number of counts inside of the circle of 
! radius "radius"
! imagePtr-pointer to an OM image
! xc, yc - coordinates of the source centre
! radius- radius od the circle 
!***********************************************************
real(single) Function  CountsInCircle(imagePtr, xc, yc, radius, maxRadius, & 
     srcNumber, psfImageTest, bkg)
    real(single), pointer :: imagePtr(:, :)
    real(single), intent(in) :: xc, yc
    real(single), intent(in) :: radius
    real(single), intent(in) :: maxRadius
    real(single), intent(in) :: bkg

    !*********************************
    ! 2011-11-02 subtracting background (bkg)

    integer, intent(in) :: srcNumber

    integer :: xl, xu, yl, yu
    real(single) :: sumPix, x, y, dx, dy, dist, fc

    integer nx, ny

    integer :: ii, jj, i1, i2, j1, j2, ic, jc, iradius 

    integer :: ii1, ii2, jj1, jj2, iic, jjc, iin, jjn 
    integer :: i, j
    integer :: i1r10, j1r10, i2r10, j2r10

    !**************************************************
    ! Test 2011-10-31
    !real (single), dimension(0 : nx - 1, 0 : ny - 1) :: image

    real(single), pointer :: psfImageTest(:, :)
    real(single) :: radius10, xc10, yc10
    real(single) :: pixArea, circleArea

    !*************************************************

    nx = size(imagePtr, 1)
    ny = size(imagePtr, 2)

    xl = 0
    yl = 0
    xu = nx
    yu = ny


    ic=int(xc)
    jc=int(yc)

    iradius=int(maxRadius*10.0+0.5)
    radius10=radius*10.0
    circleArea=PI*radius10*radius10

    iin=maxRadius*20
    jjn=maxRadius*20

    iic=iradius
    jjc=iradius

    xc10=maxRadius*10.0
    yc10=maxRadius*10.0

    i1=int(xc-radius-0.5)
    i2=int(xc+radius+0.5)
    j1=int(yc-radius-0.5)
    j2=int(yc+radius+0.5)


    i1r10=int(xc10-radius10-0.5)
    i2r10=int(xc10+radius10+0.5)
    j1r10=int(yc10-radius10-0.5)
    j2r10=int(yc10+radius10+0.5)

    !i1=ic-iradius
    !i2=ic+iradius
    !j1=jc-iradius
    !j2=jc+iradius

    fc=imagePtr(ic, jc)


    !************************************************
    !** Creating a higher resolution image (10x10 for each pixel)  
    !  2011-10-31
    !  2011-11-02 Dividing the value of each new pixel by 100
    ! to get the same number of counts as before
    !*************************************************
    !psfImageTest => allocate2dSingle(0, nx - 1, 0, ny - 1, "psfimage")
    !psfImageTest => allocate2dSingle(i1, i2, j1, j2, "psfimage")

    !psfImageTest => allocate2dSingle(0, iin, 0, jjn, "psfimage")
    !
    !psfImageTest=-1.0
    !!do ii=0,nx-1
    !!   do jj=0,ny-1
    !!do ii=i1,i2
    !!   do jj=j1,j2
    !do ii=0,iin
    !   do jj=0,jjn
    !      i=(ii-iic)/10+ic
    !      j=(jj-jjc)/10+jc
    !      if (i .gt. xl .and. i .lt. xu .and. jj .gt. yl .and. jj .lt. yu) then
    !         psfImageTest(ii,jj)= imagePtr(i, j)!/100.0
    !      endif
    !   enddo
    !enddo



    !***********************************************

    !sumPix=0.0
    !do ii=i1,i2
    !   if (ii .gt. xl .and. ii .lt. xu) then
    !      do jj=j1,j2
    !         if (jj .gt. yl .and. jj .lt. yu) then
    !            x=real(ii)
    !            y=real(jj)
    !            dx=x-xc
    !            dy=y-yc
    !            dist=sqrt(dx*dx+dy*dy)
    !            if (srcNumber .eq. 12 .and. radius .lt. 0.2) then
    !               write(*,*) "L1286 xc=", xc, " yc=", yc, " x=", x, " y=", y, " dist=", dist
    !            endif
    !            if (dist .le. radius) then
    !               sumPix=sumPix+imagePtr(ii, jj)
    !            endif
    !         endif
    !      enddo
    !   endif
    !enddo


    !********************************
    ! 2011-10-31
    sumPix=0.0
    pixArea=0.0
    do ii=i1r10,i2r10
       !if (ii .gt. xl .and. ii .lt. xu) then
          do jj=j1r10,j2r10
             !if (jj .gt. yl .and. jj .lt. yu) then
                x=real(ii)
                y=real(jj)
                dx=x-xc10
                dy=y-yc10
                dist=sqrt(dx*dx+dy*dy)
                !if (srcNumber .eq. 12 .and. radius .lt. 0.2) then
                !   write(*,*) "L1286 xc10=", xc10, " yc10=", yc10, " x=", x, " y=", y, " dist=", dist
                !endif
                if (dist .le. radius10 .and. psfImageTest(ii, jj) .ge. 0.0 ) then
                   sumPix=sumPix+psfImageTest(ii, jj)-bkg
                   pixArea=pixArea+1.0
                endif
             !endif
          enddo
      !endif
    enddo

    !if (srcNumber .eq. 12 ) then
    !   !write(*,*) "L1268 ic=", ic, " jc=", jc, " i1=", i1, " i2=", i2, &
    !   !     " j1=", j1, " j2=", j2, " fc=", fc

    !   !write(*,*) "      pixArea=", pixArea, " circleArea=", circleArea
!
!       !write(*,*) "L1422 maxRadius=", maxRadius
!       !call SaveImageTest(psfImageTest, nx, ny, "psfprofiletest0.fit", "test")
!       !call SaveImageTest2(psfImageTest, i1, i2, j1, j2, "psfprofiletest0.fit", "test")
!       call SaveImageTest2(psfImageTest, 0, iin, 0, jjn, "psfprofiletest0.fit", "test")
!!
!
!    endif

    !deallocate(psfImageTest)

    CountsInCircle=sumPix
 
end function CountsInCircle

!***********************************************************
! CountsInCircleReduced 2011-11-04 VNY
! To compute the real number of counts inside of the circle of 
! radius "radius" taking into account that the image
! is spread 100 times (10x10)
! imagePtr-pointer to an OM image
! xc, yc - coordinates of the source centre
! radius- radius od the circle 
!***********************************************************
real(single) Function  CountsInCircleReduced(imagePtr, xc, yc, radius, maxRadius, & 
     srcNumber, psfImageTest, bkg)
    real(single), pointer :: imagePtr(:, :)
    real(single), intent(in) :: xc, yc
    real(single), intent(in) :: radius
    real(single), intent(in) :: maxRadius
    real(single), intent(in) :: bkg

    integer, intent(in) :: srcNumber

    integer :: xl, xu, yl, yu
    real(single) :: sumPix, x, y, dx, dy, dist, fc

    integer nx, ny
    integer :: ii, jj, i1, i2, j1, j2, ic, jc, iradius 

    integer :: ii1, ii2, jj1, jj2, iic, jjc, iin, jjn 
    integer :: i, j
    integer :: i1r10, j1r10, i2r10, j2r10

    !**************************************************

    real(single), pointer :: psfImageTest(:, :)
    real(single) :: radius10, xc10, yc10
    real(single) :: pixArea, circleArea

    !*************************************************

    nx = size(imagePtr, 1)
    ny = size(imagePtr, 2)

    xl = 0
    yl = 0
    xu = nx
    yu = ny


    ic=int(xc)
    jc=int(yc)

    iradius=int(maxRadius*10.0+0.5)
    radius10=radius*10.0
    circleArea=PI*radius10*radius10

    iin=maxRadius*20
    jjn=maxRadius*20

    iic=iradius
    jjc=iradius

    xc10=maxRadius*10.0
    yc10=maxRadius*10.0

    i1=int(xc-radius-0.5)
    i2=int(xc+radius+0.5)
    j1=int(yc-radius-0.5)
    j2=int(yc+radius+0.5)


    i1r10=int(xc10-radius10-0.5)
    i2r10=int(xc10+radius10+0.5)
    j1r10=int(yc10-radius10-0.5)
    j2r10=int(yc10+radius10+0.5)

    fc=imagePtr(ic, jc)

    sumPix=0.0
    pixArea=0.0
    do ii=i1r10,i2r10
          do jj=j1r10,j2r10
                x=real(ii)
                y=real(jj)
                dx=x-xc10
                dy=y-yc10
                dist=sqrt(dx*dx+dy*dy)

                if (dist .le. radius10 .and. psfImageTest(ii, jj) .ge. 0.0 ) then
                   sumPix=sumPix+(psfImageTest(ii, jj)-bkg)/100.0
                   pixArea=pixArea+1.0
                endif
             !endif
          enddo
      !endif
    enddo

    CountsInCircleReduced=sumPix
 
end function CountsInCircleReduced

end module ompsfprofile1_mod





