

!************************************************
! 
!***********************************************
module ompsfprofile_mod
use dal
use cal
use errorhandling
use omconstants_mod
use omsourceparam_mod  ! In mssllib defines Struct SourceParameters
use omutility_mod
use leastsquares_mod
!use ominterpolate_mod
use omskybackground_mod
use omphotometrydefs_mod
use omphotometry_mod
use utilities_mod
use graph1_mod
use ompsf_mod
use getfilterid_mod
use gridinterpolation_mod
use ompsfprofile1_mod
use robustscale_mod
use statistics_mod
implicit none


!type(OneDSpline), private, save :: splineData
!type(PolintStruct), private, save :: polintData


integer, private :: numberOfStars = 0, totalNumberOfStars = 0

!***************************************************************
!
!
!**************************************************************


character(500), private :: plotFile = ""

logical :: pgPlotisOpened = .false.

logical, private :: plotGraph = .true.


contains

!************************************************************
!
!
!
!************************************************************
subroutine setPlotFile( file )
    character(*), intent(in) :: file
    integer :: l1, l2
    l1 = len(plotFile)
    l2 = len(trim(file) )
    l1 = min(l1, l2)
    plotFile(1 : l1) = file(1 : l1)

end subroutine setPlotFile


!****************************************************************
! 
!
! The entry point to the main program
!
!
!****************************************************************
subroutine ompsfprofile
   
    integer :: i, j, k, numImageFiles
         
    character(500) :: title
    character(5) :: filter
    real(single), pointer, dimension(:, :) :: xMean=>null(), yMean=>null(), error=>null()
    real(single), pointer, dimension(:) :: xMean1=>null(), yMean1=>null(), error1=>null()
    real(single), pointer, dimension(:, :) :: ysumMean=>null()
    real(single), pointer, dimension(:) :: ysumMean1=>null()
    real(single), pointer, dimension(:) :: psfWidth => null()
    real(single), pointer :: psfImage(:, :) => null()
    !character(256) :: psfFile
    character(1024) :: psfImageFile
    character(1024) :: psfProfileFile, psfProfileFile2 
    character(2) :: directory(1) = (/"./"/)   
    integer :: numDirectories = 0   
    integer(int16) :: nx, ny
    integer :: dotPos = 0
    integer :: length1 = 0
    integer :: length2 = 0
    character(10) :: endstring
    logical :: back = .true.
    character(3) :: numString 
    integer :: numDirectories2
    type(DataSetT) :: outPsfsSet
    character(80) :: string1
    integer :: printFlag = 0

    !write(errstr,'(a, i0)') "printFlag= ", printFlag
    !call message(AppMsg, Verbose)

    numImageFiles = parameterCount( "imagesets" )
 
    !write(*,*) "L109 numImageFiles=", numImageFiles

    psfImageFile = trim(stringParameter("psfset"))
    !write(*,*) "L113 psfImageFile=", trim(psfImageFile)

    psfProfileFile = trim(stringParameter("psfprofileset"))
    !write(*,*) "L117 psfProfileFile=", trim(psfProfileFile)

    call setPlotFile( trim(stringParameter("plotfile")) )


    filter = stringParameter("filter")
    !write(string1,*) "Processing data for the filter ", trim(filter)
    !call warning("filter", trim(string1))
    
    numDirectories = max(1, parameterCount("directorylist") )
 
    !write(*,*) "L123 numDirectories=", numDirectories

    if(numDirectories > 10) then
        plotGraph = .false.
    end if

    xMean => allocate2dSingle(1, maxNumRadii, 1, numDirectories, "xMean")    
    yMean => allocate2dSingle(1, maxNumRadii, 1, numDirectories, "yMean")
    ysumMean => allocate2dSingle(1, maxNumRadii, 1, numDirectories, "ysumMean")
    error => allocate2dSingle(1, maxNumRadii, 1, numDirectories, "error")


    psfWidth => allocateSingle(1, numDirectories, "psfWidth")

    xMean = -1.0
    yMean = -1.0
    ysumMean = -1.0
    error = -1.0

    numDirectories = 0


    !********************************
    ! 2011-05-31
    ! create the output dataset
    outPsfsSet = dataSet(psfProfileFile, create)
    call setAttribute(outPsfsSet, "CONTENT", "OM source PSF profiles")

    if(numImageFiles == 0) then
        numDirectories = parameterCount("directorylist"); 
        if(numDirectories .eq. 0) then
        
            ! the routine setStringListParameter is in param.f90 (param module)
            call setStringListParameter("directorylist", directory) 

        else   
            call modifydirectorylist();


        end if
         
        numDirectories = 0
        numDirectories2 = parameterCount("directorylist"); 
        !write(*,*) "L166 numDirectories2=", numDirectories2

        if (numDirectories2 .lt. 2) then
           printFlag=1 ! printing the FWHM for a single directory
        else
           printFlag=0 ! not printing intermediate FWHM values
        endif
        
        do i = 0, numDirectories2 - 1
            numberOfStars = 0 
        
            call loadimagefiles( i )          
            endstring="          "
            !! modify the psf profile file name to make it different for different directories
            !back=.true.
            !write(numString, '(i3.3)') numDirectories
            !dotPos=index(psfProfileFile,".",back)
            !length1=len_trim(psfProfileFile)
            !length2=length1-dotPos+1
            !endstring(1:length2)=psfProfileFile(dotPos:length1)
            !psfProfileFile2=psfProfileFile
            !if (numDirectories2 .gt. 1) then
            !   psfProfileFile2(dotPos:dotPos+2)=numString(1:3)
            !   psfProfileFile2(dotPos+3:dotPos+3+length2)=endstring(1:length2)
            !endif
            numImageFiles = parameterCount( "imagesets" )
            !write(*,*) "L188 numImageFiles=", numImageFiles
            if(numIMageFiles == 0) then
                cycle
            end if
            
            call ompsfprofile1(outPsfsSet, numDirectories,  printFlag,  &
                        xMean( :, numDirectories + 1),   &
                        yMean( :, numDirectories + 1),   &
                        ysumMean( :, numDirectories + 1),   &
                        error( :, numDirectories + 1),   &
                        psfWidth(numDirectories + 1))
           
            write(*,*) "L204 finished ompsfprofile1"
        end do        
    else
        call modifyimagelist();
        call modifysrclist();
        
        call ompsfprofile1(outPsfsSet, numDirectories, printFlag,   &
                        xMean( :, numDirectories + 1), &
                        yMean( :, numDirectories + 1), &
                        ysumMean( :, numDirectories + 1),   &
                        error( :, numDirectories + 1))

        !write(*,*) "L215 finished ompsfprofile1"
    end if

    !write(*,*) "L218 numDirectories=", numDirectories

    !if(numDirectories > 1) then
    if(numDirectories .gt. 1) then
      
        call checkWidths(numDirectories, psfWidth, xMean, yMean, error)

        xMean1 => allocateSingle(1, maxNumRadii, "xMean1")
        yMean1 => allocateSingle(1, maxNumRadii, "yMean1")
        ysumMean1 => allocateSingle(1, maxNumRadii, "ysumMean1")
        error1 => allocateSingle(1, maxNumRadii, "error1")
    

        xMean1 = -1.0
        yMean1 = -1.0
        ysumMean1 = -1.0
        error1 = -1.0


        k = maxNumRadii
        write(*,*) "L237 calling computeTheAverageProfile"
        call computeTheAverageProfile(k, xMean, yMean, ysumMean, error, &
             xmean1, ymean1, ysummean1, error1, j)

        filter = stringParameter("filter")

        write(*,*) "L243 storing the averaged profile"
        call storeAveragedProfile(outPsfsSet,filter,-1, -1, xMean1, yMean1, &
             ysumMean1, error1, title)

        write(*,*) "L247 j=", j
        if( j .gt. 10) then
        write(*,*) "L249 j is gt 10"
            !filter = stringParameter("filter")
      
            write(title, '(a, a, a, i0, a, i0, a)' ) "FILTER ", trim(filter), &
                ", AVERAGE PROFILE FROM ", numDirectories, " OBSERVATIONS (", &
                    totalNumberOfStars, " stars)"
            !write(*,*) "L244 plotting average profile"
            write(*,*) "L256 calling plotAveragedProfile"
            call plotAveragedProfile(xMean1, yMean1, ysumMean1, error1, title)

            !************************************************************
            !
            !
            !
            !***********************************************************
            if(len(trim(psfImageFile )) .gt. 3 .and. totalNumberOfStars > 0) then
   
              

                call computePSFImage(maxNumRadii, xMean1, yMean1, psfImage)
                nx = size(psfImage, 1)
                ny = size(psfImage, 2)
 
                call SaveImage( psfimage, nx, ny, trim(psfImageFile), "psf image")
                deallocate(psfImage)
            end if
        else
            call warning("No averaged profile was computed")
        end if


        call plotPsfWidths( numDirectories, psfWidth)


        deallocate( psfWidth )
        nullify( psfWidth )
       
        deallocate(xMean1, yMean1, error1)

    else
        !************************************************************
        !
        !
        !
        !***********************************************************
        if(len(trim( psfImageFile )) .gt. 3 .and. totalNumberOfStars > 0) then
            !write(*,*) "L295 computePSFImage"
            call computePSFImage(maxNumRadii, xMean(:, 1), yMean(:, 1), psfImage)
            nx = size(psfImage, 1)
            ny = size(psfImage, 2)
 
            call SaveImage( psfimage, nx, ny, trim(psfImageFile), "psf image")
            deallocate(psfImage)

        end if


     
    end if 

    call release(outPsfsSet)  

    if(totalNumberOfStars == 0) then

          call warning("No profiles were computed")
    end if


    deallocate(xMean, yMean, error)

    
    call closepgplot()

end subroutine ompsfprofile

!****************************************************************
! 
! Processes the data for a list of images in one directory
!
!
!
!****************************************************************
!subroutine ompsfprofile1(psfProfileFile, numDirectories, xDirectory, yDirectory, &
subroutine ompsfprofile1(outPsfsSet, numDirectories, printFlag, &
     xDirectory, yDirectory, ysumDirectory, &
     errorDirectory, psfWidthOut)
    integer, intent(inout) :: numDirectories
    integer, intent(inout) :: printFlag
    real(single), dimension(:) :: xDirectory, yDirectory, ysumDirectory, errorDirectory
    real(single), optional, intent(out) :: psfWidthOut
   
    character(1024) :: sourceListFile, imageFile
    ! 2011-05-31
    !character(1024), intent(in) :: psfProfileFile

    integer :: i, k, numImageFiles, j 
    character(500) :: title
    character(5) :: filter

    real(single), pointer, dimension(:, :) :: xMean=>null(), yMean=>null()
    real(single), pointer, dimension(:, :) :: ysumMean=>null(), error=>null()

    real(single), pointer, dimension(:) :: xMean1=>null(), yMean1=>null(), error1=>null()
    real(single), pointer, dimension(:) :: ysumMean1=>null()
    integer, pointer :: nRadii(:) => null()
    real(single) :: psfWidth
    character(10) :: obsId

    type(DataSetT), intent(in) :: outPsfsSet
    !character(80) :: string1

!character(500), save :: oldTitle = "xxx"
  
    numImageFiles = parameterCount( "imagesets" )

    !write(*,*) "L345 ompsfprofile1 numImageFiles=", numImageFiles

    !write(*,*) "L365 numImageFiles=", numImageFiles
    ! 2011-05-31

    !psfProfileFile="psffile001.fit"


    if(numIMageFiles .eq. 0) then
        return
    end if

    !write(*, *) "line 330- numimagefiles=", numImagefiles

    xMean => allocate2dSingle(1, maxNumRadii, 1, numImageFiles, "xMean")
    yMean => allocate2dSingle(1, maxNumRadii, 1, numImageFiles, "yMean")
    ysumMean => allocate2dSingle(1, maxNumRadii, 1, numImageFiles, "ysumMean")
    error => allocate2dSingle(1, maxNumRadii, 1, numImageFiles, "error")

    xMean = -1.0
    yMean = -1.0
    ysumMean = -1.0
    error = -1.0

    xMean1 => allocateSingle(1, maxNumRadii, "xMean1")
    yMean1 => allocateSingle(1, maxNumRadii, "yMean1")
    ysumMean1 => allocateSingle(1, maxNumRadii, "ysumMean1")
    error1 => allocateSingle(1, maxNumRadii, "error1")

    nRadii => allocateint32(1, numImageFiles, "nRadii")
    nRadii = maxNumRadii
    k = 0
  
    !***************************************************************
    !
    ! Loop through all the image files
    !
    !***************************************************************
    !********************************
    ! 2011-05-31
    ! create the output dataset
    !outPsfsSet = dataSet(psfProfileFile, create)
    !call setAttribute(outPsfsSet, "CONTENT", "OM source PSF profiles")

    do i = 1, numImageFiles

        imageFile = trim(stringParameter("imagesets", i - 1) )
        !write(*, *) i, ") L410 imagefile=", trim(imageFile)
 
        if ( .not. setExists( imageFile ) ) then
            call warning("FileNotFound", trim(imageFile))

            cycle

        end if
        if ( .not. findSrcListFile( imageFile, sourceListFile ) ) then
            call warning("SkippingFile", trim(imageFile))

            cycle
        end if
        !************************************************************
        ! nRadii stores the number of computed points for image file i
        !
        !************************************************************
        nRadii(k + 1) = maxNumRadii
        !write(*,*) "L428 calling processImage"
        call processImage( imageFile, sourceListFile, xMean1, yMean1, &
             !error1, nRadii(k + 1), psfProfileFile )
             ysumMean1, error1, nRadii(k + 1), outPsfsSet, i, numDirectories )
        
        !write(*, *) i, ") Line 125 after processimage, image=", trim(imageFile), " number of computed profiles=", nRadii(k + 1)

        !write(*,*) "L435 finished processImage"
        if(nRadii(k + 1) >= 5) then
            
            k = k + 1 
            xmean(1 : nRadii(k), k) = xMean1(1 : nRadii(k))
            ymean(1 : nRadii(k), k) = yMean1(1 : nRadii(k))
            ysummean(1 : nRadii(k), k) = ysumMean1(1 : nRadii(k))
            error(1 : nRadii(k), k) = error1(1 : nRadii(k))

        end if

    end do


    !*****************************************************************
    !
    !
    !
    !*****************************************************************
  

    !write(*,*) "L456 k=",k, " maxNumRadii=", maxNumRadii
    if(k > 1) then
        j = maxNumRadii

        if( j .ge. 10) then
           write(*,*) "L461 computeTheAverageProfile"
            call computeTheAverageProfile(j, xMean, yMean, ysumMean, error, &
                 xmean1, ymean1, ysummean1, error1, j)

        
            filter = stringParameter("filter")
         
            obsId =  getObsId(stringParameter("imagesets", 0) )
            

            write(title, '(a, a, a, a, a, i0, a, i0, a)' ) "OBSERVATION ", obsId, ", FILTER ", trim(filter), &
                ", AVERAGE PROFILE FROM ", k, " IMAGES (",  numberOfStars, " stars)"


            write(*,*) "L475 storing the averaged profile, numDirectories=", numDirectories
            call storeAveragedProfile(outPsfsSet,filter,numDirectories,printFlag,xMean1, yMean1, &
                 ysumMean1, error1, title)


            write(*, *) "Line 480 plotting averaged profile"
!write(*, *) "title=", trim(title)
      

!            if(trim(title) .eq. trim(oldTitle)) then
!                write(*, *) "old title=", trim(oldTitle)
!                write(*, *) "new title=", trim(title)
!                 call fatal("big at line 232")
!            end if
!            oldTitle = title
       
            write(*,*) "L487 calling plotAveragedProfile"
            call plotAveragedProfile(xMean1, yMean1, ysumMean1, error1, title, psfWidth)
            write(*,*) "L497 finished plotAveragedProfile"

           if(present(psfWidthOut) ) then
                psfWidthOut = psfWidth
            end if 
 

            xDirectory = xMean1
            yDirectory = yMean1
            ysumDirectory = ysumMean1
            errorDirectory = error1
            numDirectories = numDirectories + 1 
        end if  

    else if(k == 1) then


       !write(*,*) "L508 calling computeTheAverageProfile"
       j = maxNumRadii
       call computeTheAverageProfile(j, xMean, yMean, ysumMean, error, &
            xmean1, ymean1, ysummean1, error1, j)

       filter = stringParameter("filter")

       !write(*,*) "L501 storing the averaged profile, numDirectories=", numDirectories
       !write(*,*) "L516 calling storeTheAverageProfile"
       call storeAveragedProfile(outPsfsSet, filter, numDirectories, printFlag, xMean1, &
            yMean1, ysumMean1, error1, title)
         
       xDirectory = xMean1
       yDirectory = yMean1
       ysumDirectory = ysumMean1
       errorDirectory = error1
       numDirectories = numDirectories + 1 

       if(present(psfWidthOut) ) then
  
          call computeFWHM(j, xMean1, yMean1, psfWidthOut)

          

        end if


    end if

    !call release(outPsfsSet) 

    deallocate(xMean, yMean, error, nRadii)
    deallocate(xMean1, yMean1, error1)


end subroutine ompsfProfile1
!*************************************************************************
!
!
!*************************************************************************
subroutine plotAveragedProfile(x, y, ysum, error, title, fwhmOut)
   

    real(single), dimension(:) :: x, y, ysum, error
    character(*), intent(in) :: title
    real(single), optional, intent(out) :: fwhmOut
    real(single) :: xmin, xmax, fwhm
    character(200) :: text
    real(double) :: plateScale
   
    integer :: k

  

    xmin = 0.0
    xmax = real(14 / binX)

 
    

    k = count( x(2 : size(x))  > -1.0) + 1
    


    if ( .not. pgPlotIsOpened ) then

        call openPgPlot()

    end if

  
    call DrawGraph(k, x, y, error, & 
         "", "RADIUS (PIXELS)", "AZIMUTHALLY AVERAGED NORMALIZED COUNTS", &
           0.1, 0.9, 0.1, 0.8, 0.0,    &
           xmax, 0.0, 1.1, drawLines=.true.)

    call pgslw(5)
    call pgsch(1.1)
    call pgmtxt("T", 1.0, 0.5, 0.5, trim(title))


    call computeFWHM(k, x, y, fwhm)

    !*****************************************
    ! Get the Plate scale (arcsecs/pixel)
    !*****************************************
    call CAL_omGetPlateScale(0.55_double, plateScale)
    

    if(present(fwhmOut)) then
        fwhmOut = plateScale * dble(binx) * fwhm


    end if

    write(*,*) "L597 calling plotCumulativeCountsEncircled k=", k

    !-------------------------------------------------
    ! Plot the cumulative growth curve calculated from
    ! the annuli for the parameter cumulativetype=1
    ! or from the measured encircled counts for the
    ! parameter cumulativetype=2
    !-------------------------------------------------
    write(*,*) "L617 cumulativetype=",intParameter("cumulativetype") 
    if(intParameter("cumulativetype") .eq. 1) then
       call plotCumulativeCounts(k, x, y)
    else
       call plotCumulativeCountsEncircled(k, x, ysum)
    endif

    write(*,*) "L613 calling plotCalibrationProfile"
    call plotCalibrationProfile()

    write(text, '(a, f0.2, a, f0.2, a)') "Computed FWHM=", fwhm, &
            " pixels (", plateScale * dble(binx) * fwhm, " arcsecs)"

    call pgslw(3)
    call pgsls(1)
    call pgsch(1.0)
    call pgmtxt("T", -1.0, 0.1, 0.0, trim(text))

    call pgpage()
end subroutine plotAveragedProfile


!*************************************************************************
!
!
!*************************************************************************
subroutine storeAveragedProfile(outPsfsSet, filter, numDir, printFlag, &
     x, y, ysum, error, title, fwhmOut)
   

    real(single), dimension(:) :: x, y, ysum, error
    character(*), intent(in) :: title
    real(single), optional, intent(out) :: fwhmOut
    type(DataSetT), intent(in) :: outPsfsSet
    real(single) :: xmin, xmax, fwhm
    character(200) :: text
    real(double) :: plateScale
    type(TableT) :: outTable
    integer :: columnDim0(1)=(/1/)  
    integer :: k, jj
    type(ColumnT) :: radiusCol, profileCol, profileErrCol, growthCol
    real(kind=Single), pointer, dimension(:) :: radiusColumn
    real(kind=Single), pointer, dimension(:):: profileColumn
    real(kind=Single), pointer, dimension(:) :: profileErrorColumn
    real(kind=Single), pointer, dimension(:):: growthColumn
    character(5), intent(in) :: filter 
    integer, intent(in) :: numDir
    integer, intent(in) :: printFlag
    character(9) :: string1
    character(50) :: string2, string3
    xmin = 0.0
    xmax = real(14 / binX)

 
   
    k = count( x(2 : size(x))  > -1.0) + 1
    
    !write(*,*) "L622 k=", k


    !call DrawGraph(k, x, y, error, & 
    !     "", "RADIUS (PIXELS)", "AZIMUTHALLY AVERAGED NORMALIZED COUNTS", &
    !       0.1, 0.9, 0.1, 0.8, 0.0,    &
    !       xmax, 0.0, 1.1, drawLines=.true.)


    call computeFWHM(k, x, y, fwhm)


    !*****************************************
    ! Get the Plate scale (arcsecs/pixel)
    !*****************************************
    call CAL_omGetPlateScale(0.55_double, plateScale)
    
 
    if(present(fwhmOut)) then
        fwhmOut = plateScale * dble(binx) * fwhm
    end if


    write(text, '(a, f0.2, a, f0.2, a)') "Computed FWHM=", fwhm, &
            " pixels (", plateScale * dble(binx) * fwhm, " arcsecs)"
    !write(*,*) "L667 numDir=", numDir
    if (numDir .ge. 0) then
       write(string1, '(a,i2.2,a)') trim(filter),numDir,"PSF"
       write(string2, '(a,i2.2)') "Average PSF profile for dir ",numDir
       write(string3, '(a,i2.2)') "Table for the average PSF for dir ",numDir
    else
       write(string1, '(a,a)') trim(filter),"__PSF"
       write(string2, '(a)') "Average PSF profile for all data"
       write(string3, '(a)') "Table for the average PSF for all data"
    endif
    
    outTable = addTable(outPsfsSet, string1, k, string3)

    radiusCol = addColumn(outTable, "RADIUS", real32, "Pix", columnDim0, "Radial distance")
    profileCol = addColumn(outTable, "PROFILE", real32, "Mag", columnDim0, string2)
    profileErrCol = addColumn(outTable, "ERR", real32, "Mag", columnDim0, "Average PSF profile error")
    growthCol = addColumn(outTable, "GROWTH", real32, "Mag", columnDim0, string2)
    call setAttribute(outTable, "FILTER", trim(filter), "OM filter")
    call setAttribute(outTable, "PSFFWHM", fwhm, "pix", "Computed average PSF FWHM")
    call addcomment(outTable, text)

    ! printFlag < 0 corresponds to the average of all the directories 
    ! printFlag > 0 corresponds to the average of a single directory 
    if (printFlag .ne. 0) then
        write(string3, *) " FWHM = ",fwhm
        !call message(AppMsg, Verbose, string3)
        call warning("averaged profile", trim(string3))
    endif

    radiusColumn => real32data(radiusCol)
    profileColumn => real32data(profileCol)
    profileErrorColumn => real32data(profileErrCol)
    growthColumn => real32data(growthCol)

    do jj=1,k
       radiusColumn(jj-1)=x(jj)
       profileColumn(jj-1)=y(jj)
       profileErrorColumn(jj-1)=error(jj)
       growthColumn(jj-1)=ysum(jj)
    enddo
    call release(outTable)

end subroutine storeAveragedProfile

!*************************************************************************
!
!
!*************************************************************************
subroutine computeTheAverageProfile(k, xMean, yMean, ysumMean, error, &
     xmean1, ymean1, ysummean1, error1, nRadiiOut)
    integer, intent(in) :: k
    real(single), dimension(:, :) :: xMean, yMean, ysumMean, error   
    real(single), dimension(:) :: xMean1, yMean1, ysumMean1, error1
 
    integer, intent(inout) :: nRadiiOut    
    integer :: i, j, k1

    
!write(*, *) "L698 computetheaverageprofile, k=", k

    k1 = 0

    xMean1 = -1.0
    yMean1 = -1.0
    ysumMean1 = -1.0
    do i = 1, k      
        j = count( xmean(i, : ) >= 0.0)
         
        if(j == 0) then
            cycle
        end if

        k1 = k1 + 1
        xmean1(k1) = xMean(i, 1)
  
 !       call ComputeAverage(yMean(i, 1 : j), error(i, 1 : j), j, ymean1(k1), error1(k1) )

        !call computetheMean(j, yMean(i, 1 : j), error(i, 1 : j), &
        !         ymean1(k1), error1(k1))

        call computeTheMean2(j, yMean(i,1:j), ysumMean(i,1:j), error(i, 1 : j), &
             yMean1(k1), ysumMean1(k1), error1(k1))  


        !write(*, *) k1, ") x=", xmean1(k1), " y=", ymean1(k1)          



    end do

    nRadiiOut = k1


end subroutine computeTheAverageProfile


!*******************************************************
! Given the name of an image file returns the name
! of the corresponding srclist file
!
!*******************************************************
logical function findSrcListFile( imageFile, srcListFile )
    character(*), intent(in) :: imageFile
    character(1024), intent(out) :: srcListFile
    integer(int16) :: numSrcListFiles, i, length
    character(10) :: expId1, expId2
    
    expId1 = getExpId( imageFile )

    !write(*,*) "L648 expId1=", expId1

    numSrcListFiles = parameterCount( "srclistsets" )
    !write(*,*) "L651 numSrcListFiles=", numSrcListFiles
     
    do i = 1, numSrcListFiles

        srcListFile = trim(stringParameter("srclistsets", i - 1) )
  
        if ( .not. setExists( srcListFile ) ) then
          
           cycle

        end if
        expId2 = getExpId( srcListFile )
        if(expId1 == expId2) then
            
            findSrcListFile = .true.
            return
        end if


    end do

    !*************************************************************
    !
    ! No source-list files passed through parameter srclistsets
    !
    !*************************************************************
    
    length = len(trim(imageFile ))
    if(length .lt. 31) then
        call warning("FileNotFound", trim(srcListFile))
        findSrcListFile = .false.
        return

    end if
 
    srcListFile = imageFile
    !write(*,*) "L687 srcListFile=", srcListFile

    
    srcListFile(length - 13 : length - 8) = "SWSRLI"
    !write(*,*) "L691 srcListFile=", trim(srcListFile)

    if ( .not. setExists( trim(srcListFile) ) ) then
        call warning("FileNotFound", trim(srcListFile))
        findSrcListFile = .false.

    else
        findSrcListFile = .true.

    end if           

end function findSrcListFile
!***********************************************************
!
!
!**********************************************************
character(10) function getObsId( file )
    character(*), intent(in) :: file
    type(DataSetT) :: dataSetH

    dataSetH = dataSet( trim(file), READ)

    getObsId = stringAttribute( dataSetH, "OBS_ID")
  

    call release(dataSetH)
  

end function getObsId

!***********************************************************
!
!
!**********************************************************
character(10) function getExpId( file )
    character(*), intent(in) :: file
    type(DataSetT) :: dataSetH

    dataSetH = dataSet( trim(file), READ)

    getExpId = stringAttribute( dataSetH, "EXP_ID")
  

    call release(dataSetH)
  

end function getExpId


!***********************************************
!
!
!
!***********************************************
!subroutine processImage( imageFile, sourceListFile, xMean, yMean, error, nRadii, psfProfileFile )
subroutine processImage( imageFile, sourceListFile, xMean, yMean, ysumMean, error, &
     nRadii, outPsfsSet, imageNum, dirNum )
    character(*), intent(in) :: imageFile, sourceListFile
    integer, intent(in) :: imageNum
    integer, intent(in) :: dirNum

    ! 2011-05-31
    !character(*), intent(in) :: psfProfileFile
    type(DataSetT), intent(in) :: outPsfsSet
    type(TableT) :: outTable
    type(ColumnT) :: radiusCol, profileCol, profileErrCol, growthCol
    type(ColumnT) :: srcnumCol, xCol, yCol, maxCol, totalCol
    integer :: columnDim0(1)=(/1/)
    integer :: columnDim(1)

    integer(int32), pointer, dimension(:) :: srcnumberColumn
    real(kind=Single), pointer, dimension(:) :: xColumn
    real(kind=Single), pointer, dimension(:) :: yColumn
    real(kind=Single), pointer, dimension(:) :: maxColumn
    real(kind=Single), pointer, dimension(:) :: totalColumn
    real(kind=Single), pointer, dimension(:) :: radiusColumn
    real(kind=Single), pointer :: profileColumn(:)
    real(kind=Single), pointer, dimension(:) :: profileErrorColumn
    real(kind=Single), pointer :: growthColumn(:)
    real(kind=Single) :: radius
    integer(int32) :: iorderedNumber

   ! 2011-05-31

    real(single), pointer, dimension(:) :: xMean, yMean, ysumMean, error
    integer, intent(inout) :: nRadii
   
    type(SourceParameters), pointer    :: sources(:) => null()            ! The input test sources
    type(datasetT) :: dataSetH
    integer(int16) :: nx, ny
    integer :: k, nRadiiOut
       
    real(single) :: sigma
    logical :: binBpe
    type(profileData), pointer :: data(:) => null()
    character(500) :: title
    character(5) :: filter

    integer :: ii, jj
    character(11) :: string1
    integer :: i

    dataSetH = dataSet( imageFile, READ)

    ! compute binning
    binx = int32Attribute( dataSetH, "BINAX1")
    biny = int32Attribute( dataSetH, "BINAX2")
    binBPE = booleanAttribute( dataSetH, "BINBPE")

    binx = 2 ** binx
  
    if (binBPE) then
        binx = binx * 2
     
    end if

    !************************************************************
    !
    !
    !***********************************************************
    if( binX .ne. intParameter("binning") ) then
        call warning("IgnoringImageFile", trim(imageFile))
        nRadii = 0
        return
    end if


    call GetImage(dataSetH, nx, ny, theimage)

    filter = trim( stringAttribute(dataSetH, "FILTER") )
    call CAL_setState(OM, filterId = filterIdToInt(filter) )   
    call release(dataSetH)
  
    title = trim(imageFile)
    k = len(trim(title))
    if( title(1 : 1) .eq. ".") then
        title(1 : k - 2) = title(3 : k)

    end if    

    write(title, '(a, a, a)' ) trim(title), " Filter ", trim(filter)

    !write(*, *) "L853 title=", trim(title)


    backgroundCountsMode = ComputeBackgroundMode2(theimage, nx, ny, 3.0, sigma)
!    write(*, *) "Global background mode=", backgroundCountsMode



    call loadSources(sources, sourceListFile, -1_int8, -1_int8)


    call initialiseArrays()

    if(associated(sources)) then
        allocate(data(1 : size(sources)))
     
    !write(*,*) "L942----------------------loadSources, size=", size(sources)
    !do i = 1, size(sources)
    !   
    !   write(*, *) "L946 i= ", i, " confusion flag=", sources(i)%confusionFlag
    !enddo

        call starprofiles(sources, theImage, data, k)
       

        !write(*, *) "L876  Number of computed star profiles=", k     
        !write(*,*) "L877 filter=", trim(filter)

        write(errstr,'(a, i0, a, a)') "Number of computed source profiles ", &
             k, " Filer=", trim(filter)
        call message(AppMsg, Verbose)



        if(k > 0) then

           !********************************
           ! 2011-05-31
           ! create the output dataset
           !outPsfsSet = dataSet(psfProfileFile, create)
           !call setAttribute(outPsfsSet, "CONTENT", "OM source PSF profiles")
           !write(*,'(a,i3.3)') " imageNum=",imageNum
           !write(*,'(a,i2.2)') " dirNum=",dirNum

           !write(*,*) "L1037 ---------- After starprofiles k=", k
           !do i = 1, k
           !   ii=data(i)%sourceNumber
           !   write(*,*) " i=", i, " x=", sources(ii)%xpos, " y=", sources(ii)%ypos, &
           !        " countsMax=", data(i)%countsMax, " total=", data(i)%totalCounts
           !enddo


           write(string1,'(a,i2.2,i3.3)') filter,dirNum,imageNum
           outTable = addTable(outPsfsSet, string1, k, "Table with the source PSF profiles")

           !Adding columns to the output table
           columnDim(1)=nRadii
           srcnumCol = addColumn(outTable, "SRCNUM", integer32, " ", columnDim0, "Source number")
           xCol = addColumn(outTable, "XPOS", real32, "Pix", columnDim0, "Source x-position")
           yCol = addColumn(outTable, "YPOS", real32, "Pix", columnDim0, "Source y-position")
           maxCol = addColumn(outTable, "MAX", real32, "Counts", columnDim0, "Source max value")
           totalCol = addColumn(outTable, "TOTAL", real32, "Counts", columnDim0, "Source total counts")
           radiusCol = addColumn(outTable, "RADIUS", real32, "Pix", columnDim, "Radial distance")
           profileCol = addColumn(outTable, "PROFILE", real32, "Mag", columnDim, "PSF profile")
           profileErrCol = addColumn(outTable, "ERR", real32, "Mag", columnDim, "PSF profile error")
           growthCol = addColumn(outTable, "GROWTH", real32, "Mag", columnDim, "Cumulative growth curve")

           call addcomment(outTable, title)

           srcnumberColumn=>int32data(srcnumCol)
           xColumn => real32data(xCol)
           yColumn => real32data(yCol)
           maxColumn => real32data(maxCol)
           totalColumn => real32data(totalCol)
           write(*,*) "----------------"
           do jj=1,k
              iorderedNumber=data(jj)%sourceNumber
              !if (iorderedNumber .eq. 12) then
              !write(*,*) "L1061 jj=", jj, " source=", iorderedNumber, " srcNumber=", &
              !     sources(iorderedNumber)%number, " x=", sources(iorderedNumber)%xPos, &
              !     " y=", sources(iorderedNumber)%yPos, " max=", data(jj)%countsMax
              !   write(*,*) "----------------"
              !endif
              srcnumberColumn(jj-1)=sources(iorderedNumber)%number
              xColumn(jj-1)=sources(iorderedNumber)%xPos
              yColumn(jj-1)=sources(iorderedNumber)%yPos
              maxColumn(jj-1)=data(jj)%countsMax
              totalColumn(jj-1)=data(jj)%totalCounts
              radiusColumn => real32VectorData(radiusCol,jj-1)
              profileColumn => real32VectorData(profileCol,jj-1)
              profileErrorColumn => real32VectorData(profileErrCol,jj-1)
              growthColumn => real32VectorData(growthCol,jj-1)
              do ii = 1, nRadii
                 radius = real(ii - 1) / real(binx * 4)
                 radiusColumn(ii-1)=radius
                 profileColumn(ii-1)=data(jj)%f(ii)
                 profileErrorColumn(ii-1)=data(jj)%df(ii)
                 growthColumn(ii-1)=data(jj)%sumf(ii)
                 !if (iorderedNumber .eq. 12) then                
                 !   write(*, *) ii, ") radius=", radiusColumn(ii-1), " f=", profileColumn(ii-1), &
                 !        " sumf=", growthColumn(ii-1)
                 !endif
              enddo

           enddo
           call release(outTable)
           !call release(outPsfsSet)

           ! 2011-05-31
           !********************************************


            numberOfStars = numberOfStars + k
             
            totalnumberOfStars = totalNumberOfStars + k
      
            !**************************************************************************
            !
            !
            !
            !**************************************************************************
            !write(*,*) "L1023 before ComputeAverageProfile"
 !            write(*, *) "processimage line 427- calling averageprofile, nradii=", nradii

            !write(*, *) "L851 nradii=", nRadii, " k=", k
            !write(*,*) "----------------"
            !do jj=1,k
            !   write(*,*) "L855 source=", jj
            !   write(*,*) "----------------"
            !   do ii = 1, nRadii
            !      write(*, *) ii, ") f=", data(jj)%f(ii) , " df=", data(jj)%df(ii)
            !   enddo
            !enddo
            !write(*,*) "----------------"
            !write(*,*) "L1035 before ComputeAverageProfile"

            !call computeAverageProfile(data, k, xMean, yMean, error, nRadii, nRadiiOut)
            call computeAverageProfile2(data, k, xMean, yMean, ysumMean, error, nRadii, nRadiiOut)
            nRadii = nRadiiOut     
!            write(*, *) "processimage line 427 nradii=", nRadii, " nradiiout=", nRadiiOut  
            !write(*, *) "L1068 calling plotstarprofiles"
            if( plotGraph) then
               call plotStarProfiles(data, k, trim(title))
            end if

        else

           nRadii = 0         
        end if
        call deleteArrays()
     
        deallocate(sources, data)
    else
        nRadii = 0
        

    end if
   
    call deleteArrays()
   
    deallocate(theimage)

   

end subroutine processImage

!***************************************************************
!
!
!
!
!
!**************************************************************
subroutine plotStarProfiles(data, ndata, title)
    type(profileData), pointer :: data(:)
    integer, intent(in) :: nData
    character(*), intent(in) :: title
    real(single), pointer, dimension(:) :: x=>null(), y=>null(), ysum=>null(), error=>null()
    real(single) :: fMax, xmax, fwhm
    real(double) :: plateScale
    integer :: i, j, sourceNumber
    character(100) :: text
      
    integer :: nRadii, nRadiiOut
   

    j = size(data(1)%radius)

    x => allocateSingle(1, j, "x")
    
    x = data(1)%radius

    y => allocateSingle(1, j, "y")
    ysum => allocateSingle(1, j, "ysum")
    error => allocateSingle(1, j, "error")

!write(*, *) "plotstarprofiles line 979- title=", trim(title), " ndata=", ndata

    fMax = 0.0
    xmax = 0.0
    do i = 1, nData
        fMax = max(fMax, maxval(data(i)%f))
        xmax = max(xmax, maxval(x))

    end do
   
    xmax = real(14 / binx)

 
    !***********************************************************************************
    !
    ! Plot of test rate against detectes/test rate
    !
    !**********************************************************************************
  
    if ( .not. pgPlotIsOpened ) then

        call openPgPlot()

    end if

    call DrawGraph(0, x, y,       & 
         trim(title), "RADIUS (PIXELS)", "AZIMUTHALLY AVERAGED NORMALIZED COUNTS", &
           0.1, 0.9, 0.1, 0.8, 0.0, xmax, 0.0, 1.1)

    call pgsch( symbolSize )
    call pgsci(blue)

    do i = 1, nData
        j = data(i)%size
        x(1 : j) = data(i)%radius(1 : j) 
              
        y(1 : j) = data(i)%f(1 : j) 
!        call pgpoint( j, x, y, symbol( 1 ) )
        sourceNumber = data(i)%sourceNumber
        write(text, '(i0)') sourceNumber
!        call pgptxt(x(j), y(j), 0.0, 0.0, text)
!        write(*, *) i, ") j=", j, " minval(y)=", minval(y(1:j)), " max=", maxval(y(1:j))
      
    end do    


    nRadii = maxNumRadii

    x = -1.0

    !write(*,*) "L1141 before ComputeAverageProfile"

    !call computeAverageProfile(data, ndata, x, y, error, nRadii, nRadiiOut)

    call computeAverageProfile2(data, ndata, x, y, ysum, error, nRadii, nRadiiOut)


    call pgsci(red)
    call pgsch(1.0)
    if(nData .eq. 1) then

        write(text, '(a, i0, a)') "Red points from 1 source only" 
    else
        write(text, '(a, i0, a)') "Red points are the average of ", nData, " sources" 

    end if

    call pgmtxt("T", -1.0, 0.02, 0.0, trim(text))

    call computeFWHM(nRadiiOut, x, y, fwhm)

    !*****************************************
    ! Get the Plate scale (arcsecs/pixel)
    !*****************************************
    call CAL_omGetPlateScale(0.55_double, plateScale)
    
    call pgsci(black)

    write(text, '(a, f0.2, a, f0.2, a)') "Computed FWHM=", fwhm, &
             " pixels (", plateScale * dble(binx) * fwhm, " arcsecs)"

    call pgmtxt("T", -2.0, 0.013, 0.0, trim(text))
    call pgsci(red)

    call pgsch(0.75)
    call pgslw(10)
    call pgpt( nRadiiOut, x, y, -1 )
    call pgslw(2)
    call pgsci( black )
    call drawline( nRadiiOut, x, y)


    call pgslw(1)
  
    call pgerry( nRadiiOut,x,  y - error, y + error, 2.0)   

    !-------------------------------------------------
    ! Plot the cumulative growth curve calculated from
    ! the annuli for the parameter cumulativetype=1
    ! or from the measured encircled counts for the
    ! parameter cumulativetype=2
    !-------------------------------------------------
    !write(*,*) "L1254 cumulativetype=",intParameter("cumulativetype") 
    if(intParameter("cumulativetype") .eq. 1) then
       call plotCumulativeCounts(nRadiiOut, x, y)
    else
       call plotCumulativeCountsEncircled(nRadiiOut, x, ysum)
    endif

    call plotCalibrationProfile()

    call pgpage

end subroutine plotStarProfiles



!*******************************************************************
!
!
!****************************************************************
subroutine openPgPlot

    call pgbegin(0, trim(plotfile) // '/CPS', 1, 1)    
    call SetUpDefaultColourIndices( )



    pgPlotisOpened = .true.

end subroutine openPgPlot
!**************************************************************
!
!
!
!**************************************************************
subroutine closePgPlot

  

    if ( pgPlotisOpened ) then
        call pgclos()

    end if
end subroutine closePgPlot
!*****************************************************************
!
!
!
!*****************************************************************
subroutine plotCalibrationProfile()
    real(single) :: rStart, rFinish

    real(double) :: dr

    integer :: i
    integer, parameter :: nIntervals = 1001
    real(single), pointer, dimension(:) :: x=> null(), y=>null()
    real(single) :: radius

    rStart = 0.0
    rFinish = real(20 / binx)

    x => allocateSingle(1, nIntervals, "x")
    y => allocateSingle(1, nIntervals, "y")



    dr = (dble(rFinish) - dble(rStart) ) / dble(nIntervals)


    do i = 1, nIntervals

        x(i) = dble(rStart) + dble(i - 1) * dr
        radius = dble(x(i)) * dble(binx)        
        y(i) =  PSFFractionOfTotalCounts(1024.5, 1024.5, radius, 0.5)

    end do
   
    call pgsls(2)
    call pgslw(2)
    call pgsci( green )
    call drawline( nIntervals, x, y)

    call pgslw(1)
    call pgsls(1) 
 
    call pgsch(1.0)
    call pgslw(3)
 
    call pgmtxt("T", -4.0, 0.80, 1.0, "Calibration curve")

    call pgsci(black)
    deallocate(x, y)

end subroutine plotCalibrationProfile



!******************************************************************
!
!
!*****************************************************************
subroutine plotCumulativeCounts(nData, x, y)
    integer, intent(in) :: nData
    real(single), dimension(:) :: x, y
  
    real(double) :: xmin, xmax, dx, xl, xu, xc, f, fTotal, fTotal1
    integer :: i, ndata1
    integer :: size = 100
    real(single), pointer, dimension(:) :: xData => null(), yData => null()
    character(100) :: text

    real(double) :: fprev
    real(double) :: ringArea

    nData1 = count(x(1 : nData)  >= 0.0)
    !**********************************************************
    !
    !
    !**********************************************************
    call ConstructSpline(nData1, x, y, splineData)
    !write(*,*) "L1287 plotting cumulative counts, size=", size, " PI=", PI
    size = 100
    xData => allocateSingle(1, size, "xData")
    yData => allocateSingle(1, size, "yData")

    xmin = 0.0_double
    xmax = min( dble(12 / binx), dble(x(ndata1)))
    dx = (xmax - xmin) / dble(size)
   

    xl = 0.0_double
    xu = xmax
    fTotal = 0.0_double
    ftotal1 = 0.0_double 

    fprev=0.0_double
    !write (*,*) "L1346 integrated curve:"
    !write(*,*) "-----------------------------"
    do i = 1, size
        xl = xmin + dble(i - 1) * dx
        xu = xmin + dble(i) * dx
        xc=(xl + xu) / 2.0_double

        ringArea=2.0_double*PI*xc*dx


        f = QGauss(xl, xu, cumulativeCountsFunction) 

!******************************************
! 2011-10-06 vny
!        fTotal = fTotal + f
        !fTotal=fTotal+fprev
        fTotal=fTotal+f*ringArea
        fprev=f*ringArea
!*************************************
        !if(fTotal .lt. fTotal1) then
        !    fTotal = fTotal1
        !    size = i - 1
        !    exit
        !end if
 
       fTotal1 = fTotal
      
        xData(i) = (xl + xu) / 2.0_double
        !xData(i) = xu

        yData(i) = fTotal
 
    
!       write(*, *) i, ") x=", xData(i), " f=", yData(i), " ringArea=", &
!            ringArea

    end do
   
    ydata = yData / fTotal

!*****************************************
!    write (*,*) "L1346 integrated curve:"
!    write(*,*) "-----------------------------"
!
!    do i = 1, size      
!       write(*, *) i, ") x=", xData(i), " f=", yData(i), " ringArea=", ringArea
!    end do
!*****************************************


    call pgsci(blue)
!    call pgpoint( size, xData, yData, 2 )


   
    call pgsch(1.0)
    call pgslw(3)
    write(text, '(a)') "Fraction of total counts" 
    !call pgmtxt("T", -4.0, 0.95, 1.0, trim(text))
    call pgmtxt("T", -2.0, 0.90, 1.0, trim(text))


!    call pgsci(black)
    call pgsls(1)
     
    call drawline(size, xData, yData)
    call pgslw(1) 

    call pgsci(black)

    call pgsls(2)
    call pgmove(0.0, 1.0)
    call pgdraw(100.0, 1.0)
    call pgsls(1)

    call DestructSpline( splineData )

    deallocate(xData, yData)


end subroutine plotCumulativeCounts

!******************************************************************
!
!
!*****************************************************************
subroutine plotCumulativeCountsEncircled(nData, x, y)
    integer, intent(in) :: nData
    real(single), dimension(:) :: x, y
  
    real(double) :: xmin, xmax, dx, xl, xu, xc, f, fTotal, fTotal1
    integer :: i, i1, ndata1, ixl, ixu
    integer :: size = 100
    real(single), pointer, dimension(:) :: xData => null(), yData => null()
    character(100) :: text

    real(double) :: fprev
    real(double) :: ringArea


    nData1 = count(x(1 : nData)  >= 0.0)

    write(*,*) "L1473 plotting cumulative counts, size=", size, " ndata=", nData
    !do i=1,nData
    !   write(*,*) "l1474 i=", i, " x=", x(i), " y=", y(i)!, " spline=", splineData%y(i)
    !enddo
    !**********************************************************
    !
    !
    !**********************************************************
    call ConstructSpline(nData1, x, y, splineData)

    
    !do i=1,nData
    !   write(*,*) "l1474 i=", i, " x=", x(i), " y=", y(i), " spline=", splineData%y(i)
    !enddo

    size = 100
    xData => allocateSingle(1, size, "xData")
    yData => allocateSingle(1, size, "yData")

    xmin = 0.0_double
    xmax = min( dble(12 / binx), dble(x(ndata1)))
    dx = (xmax - xmin) / dble(size)
   
    !write(*,*) "L1505 xmin=", xmin, " xmax=", xmax

    xl = 0.0_double
    xu = xmax
    fTotal = 0.0_double
    ftotal1 = 0.0_double 

    fprev=0.0_double
    
    !write (*,*) "L1346 integrated curve:"
    !write(*,*) "-----------------------------"
    do i = 1, size
        xl = xmin + dble(i - 1) * dx
        xu = xmin + dble(i) * dx
        xc=(xl + xu) / 2.0_double

        !ringArea=2.0_double*PI*xc*dx


        !f = QGauss(xl, xu, cumulativeCountsFunction) 

!******************************************
! 2011-11-01 interpolate the curve

        do i1=1,nData-1
           fTotal=1.0
           if (xc .ge. x(i1) .and. xc .lt. x(i1+1)) then
              fTotal=splineData%y(i1)+(splineData%y(i1+1)-splineData%y(i1))*(xc-x(i1))/(x(i1+1)-x(i1))
              exit
           endif
        enddo
            
        xData(i) = xc        
        yData(i) = fTotal
 
    end do
   

    call pgsci(blue)
!    call pgpoint( size, xData, yData, 2 )


   
    call pgsch(1.0)
    call pgslw(3)
    write(text, '(a)') "Fraction of total counts" 
    !call pgmtxt("T", -4.0, 0.95, 1.0, trim(text))
    call pgmtxt("T", -2.0, 0.90, 1.0, trim(text))


!    call pgsci(black)
    call pgsls(1)

    !write(*,*) "L1581 drawing a line"
    call drawline(size, xData, yData)
    call pgslw(1) 

    call pgsci(black)

    call pgsls(2)
    call pgmove(0.0, 1.0)
    call pgdraw(100.0, 1.0)
    call pgsls(1)

    call DestructSpline( splineData )

    deallocate(xData, yData)


end subroutine plotCumulativeCountsEncircled

!***********************************************************************
!
!
!***********************************************************************
subroutine plotPsfWidths( size, psfWidth)
    integer, intent(in) :: size
    real(single) :: psfWidth(:)
    real(single) :: binWidth
    character(200) :: title = "test", xlabel = "FWHM (arcsecs)"
    character(200) :: text
    character(5) :: filter
    real(single) :: xMin, xmax, median, p1, p2


    if ( .not. pgPlotIsOpened ) then

        call openPgPlot()

    end if



    filter = stringParameter("filter")
      




    xMin = floor( minval(psfWidth(1 : size)) * 10.0_double) / 10.0_double
    xMax = ceiling( maxval(psfWidth(1 : size)) * 10.0_double) / 10.0_double


    if( (xMax - xMin) .le. 1.0) then
        binWidth = 0.05
    else
        binWidth = 0.1
    end if
    write(title, '(a, i0, a)') "PSF FWHM DISTRIBUTION (", size, " OBSERVATIONS)"
    call DrawHistogram( size, psfWidth, binWidth, &
                 trim(xLabel), trim(title), 0.1, 0.9, 0.1, 0.7, &
                 xMin, xMax)


    !**********************************************************************
    !
    !
    !*********************************************************************
    p1 = Percentile(size, psfWidth, 0.05)
    p2 = Percentile(size, psfWidth, 0.95)
    median = MedianValue(psfWidth, size)
  
    write(text, '(a, a, a, f0.2, a, f0.2, a, f0.2, a, i0)') "Filter ", trim(filter), &
                 " 0.05 percentile=", p1, &
          ", 0.95 percentile=", p2, " median=", median, " N=", size

  
    call pgslw(3)
    call pgsch(1.1)
    call pgmtxt("T", -1.0, 0.01, 0.0, trim(text))

 
    call pgpage()

end subroutine plotPsfWidths
!********************************************************************
!
!
!*******************************************************************
subroutine checkWidths(numDirectories, psfWidth, xMean, yMean, error)
    integer, intent(inout) :: numDirectories
    real(single), pointer :: psfWidth(:)
    real(single), pointer, dimension(:, :) :: xMean, yMean, error
    real(single) :: snValue, median, std
    integer :: i, j, k, minCount

    if(numDirectories < 8) then
        return
    end if

    minCount = max(5, numDirectories / 2)

  
    write(errstr,'(a, i0)') "Checking FWHMs, N=", numDirectories
    call message(AppMsg, Verbose)



    do j = 1, 3
        median = MedianValue(psfWidth, numDirectories)
        snValue = max(0.01, sn(psfWidth, numDirectories))
        std = 2.5 * snValue
        
        write(errstr,'(a, i0, a, f0.2, a, f0.2, a, i0)') "Iteration ", j, " - median=", median, &
              " SN value=", snValue, " N=", numDirectories
        call message(AppMsg, Verbose)



        k = 0
        do i = 1, numDirectories
            if (abs( psfWidth(i) - median) .le. std) then
                k = k + 1
                psfWidth(k) = psfWidth(i)
                xMean(:, k) = xMean(:, i)
                yMean(:, k) = yMean(:, i)
                error(:, k) = error(:, i)
             end if
        end do

      
        if(k ==  numDirectories .or. k < minCount) then
            exit
        end if
        numDirectories = k 

    end do

    write(errstr,'(a, f0.2, a, i0)') "Median psf FWHM (arcsecs)=", median, " N=", numDirectories
    call message(AppMsg, Verbose)
   
    median = MedianValue(psfWidth, numDirectories)
 
    write(errstr,'(a, f0.2, a, i0)') "After clipping, Median psf FWHM (arcsecs)=", median, " N=", numDirectories
 

end subroutine checkWidths






end module ompsfprofile_mod





