
!******************************************
! Test program for omdetect
!******************************************
module utilities_mod

use dal
use cal
use omsourceparam_mod  ! In mssllib defines Struct SourceParameters
use omutility_mod
use param

contains



!*********************************************************************************
! subroutine loadSources
! Info: loads sources found in dataset(setName), and returns a newly created vector
!       of sources.
! 
!********************************************************************************
subroutine loadSources(sources, setName, extensionFlag, confusionFlag)

    type(SourceParameters), pointer    :: sources(:)
    character(*), intent(in) :: setName
    integer(int8), intent(in) :: extensionFlag, confusionFlag
 
    type(SourceParameters), pointer    :: tempSources(:) => null()
    type(ColumnT) :: columnH 
!    real(single), intent(out) :: background
  
    ! locals
    integer(int32)        :: nSources
  
    type(DataSetT)        ::  set
    type(TableT)          :: inTableH

    integer i, count         
   


      ! --- don't bother if none available
    if (trim(setName) == "" ) return

    ! Read in the OM OSW Source List file.
    set = dataSet(setName, read)


write(*, *) "Loadsources loading sources from ", trim(setName)      

    if (.not. hasBlock(set, "SRCLIST")) then
    

        ! Incompatible file - can't continue
        write (errstr, *) "Could not find any compatibile blocks in set ", trim(setName)
        call fatal("BadFileType")

    else
        ! Get a handle on the source list binary table extension.
        inTableH = table(set, "SRCLIST")

    end if

    if (associated(sources)) deallocate(sources)
    nullify(sources)     
    
  
                 
    ! Get the number of rows in the source list binary table extension.
    nSources = numberOfRows(inTableH)

    !write(*,*) "L73 nSources=", nSources

    if (nSources == 0) then
      

        call release(inTableH)
        call release(set)
      
        return

    end if



   
    allocate(tempSources(1 : nSources))
   
 
    ! Get the data for each source from the source list file

    call LoadInt32Column(inTableH, "SRCNUM", tempSources%number)
  
    call LoadReal32Column(inTableH, "XPOS",  tempSources%xPos)
 
  
    call LoadReal32Column(inTableH, "YPOS",  tempSources%yPos)
    call LoadReal32Column(inTableH, "POSERR",  tempSources%poserr)


 
   
  
    call LoadReal32Column(inTableH, "RATE", tempSources%rawCountRate)
 
  
    call LoadReal32Column(inTableH, "RATE_ERR", tempSources%rawCountRateErr)
 

    call LoadReal32Column(inTableH, "CORR_RATE", tempSources%corrCountRate)
  
  
    call LoadReal32Column(inTableH, "CORR_RATE_ERR", tempSources%corrCountRateErr)
 
  
  

    call LoadReal32Column(inTableH, "BACKGROUND_RATE",  tempSources%rawBackgroundCountRate)
  
    call LoadReal32Column(inTableH, "CORR_BACKGROUND_RATE",  tempSources%corrbackgroundCountRate)
  

    call LoadReal32Column(inTableH, "MAG", tempSources%magnitude)
  
  
    call LoadReal32Column(inTableH, "MAGERR", tempSources%magnitudeErr)
  
  
    call LoadReal32Column(inTableH, "SIGNIFICANCE", tempSources%significance)
    
  
    call LoadReal32Column(inTableH, "APERTURE", tempSources%apertureRadius)


    call LoadInt8Column(inTableH, "EFLAG", tempSources%extensionFlag)
    
  
  

    ! Get the RAS and decs if they exist
    call LoadReal64Column(inTableH, "RA", tempSources%rightAscension)

  
    call LoadReal64Column(inTableH, "DEC", tempSources%declination)
  
  
    columnH = column(inTableH, "QFLAG")

    if (dataType(columnh) .eq. integer16) then
        call LoadInt16Column(inTableH, "QFLAG", tempSources%qualityFlag)
    else
        call LoadInt8Column(inTableH, "QFLAG", tempSources%confusionFlag)
        tempSources%qualityFlag =  int(tempSources%confusionFlag, int16)  
    end if
    call release(columnH)


    call LoadInt8Column(inTableH, "CFLAG", tempSources%confusionFlag)


!write(*, *) "utilities_mod.f90 Line 324 load sources loaded confusion flags"
!do i = 1, nsources
!   write(*, *) i, " confusion flag=", tempsources(i)%confusionFlag
!end do
 

    call release(inTableH)
    call release(set)


!write(*, *) "lINE 324 EXTENSION flag=", extensionFlag, " confusionflag=", confusionFlag

    if (extensionFlag .lt. 0 .and. confusionFlag .lt. 0) then
        allocate(sources(1 : nSources))
        !write(*,*) "L176 extension or confusion flag <0"
        sources = tempSources

    else   ! Need to filter source-list
 
       !write(*,*) "L181 needs to filter out the source list"
        count = 0
        do i = 1, nSources

!write(*, *) i, " confusion flag=", tempsources(i)%confusionFlag
           if ( (extensionFlag .eq. tempSources(i)%extensionFlag .or. &
                extensionFlag .lt. 0)  .and. &
                     (tempSources(i)%confusionFlag .le. confusionFlag &
             .or. confusionFlag .eq. -1_int8)) then
                  
               count = count + 1
           end if
      
        end do

!write(*, *) "Line 374 count=", count


    
        allocate(sources(1 : count))
        count = 0
        do i = 1, nSources
           
            if ( (extensionFlag .eq. tempSources(i)%extensionFlag .or. &
                  extensionFlag .lt. 0) .and. &
                     (tempSources(i)%confusionFlag .le. confusionFlag &
                     .or. confusionFlag .eq. -1_int8)) then
         
            
                count = count + 1
                sources(count) = tempSources(i)
       
            end if


        end do
    end if



    deallocate(tempSources)
    nullify(tempSources)

end subroutine loadSources

!***********************************************
!
!***********************************************
subroutine AddMagnitudeColumnsToTable(srcListFile, source)     
        
    character(*), intent(in) :: srcListFile
    type(SourceParameters) :: source(:) 
    type(DataSetT) ::  set
    type(TableT) :: tableSet
    set = DataSet(srcListFile, modify)
    tableSet = Table(set, "SRCLIST", modify)

    call AddReal32Column(tableSet, source%magnitude, size(source), "MAG", &
        "mag", "Instrumental magnitude", -999.9)

    call AddReal32Column(tableSet, source%magnitudeErr, size(source), "MAGERR", &
        "mag", "Instrumental magnitude error", -999.9)


    call release(tableSet)
    call release(set)

end subroutine  AddMagnitudeColumnsToTable


end module utilities_mod













